
import cv2
import numpy as np
import glob
import os

# extract images
def extract_images(folder_path):
  data_path = os.path.join(folder_path,'*g') 
  files = glob.glob(data_path) 
  imgfiles=[]
  for files in files:
    if (files.endswith(".jpeg")==True)|(files.endswith(".jpg")==True):
      imgfiles.append(files)
  return imgfiles
#%%
# smoothing
# smoothing
def image_smoothing_opencv(datapath, is_image, method='average', show_image=False,save_image=True):
  # if user selects a invalid method we raise an error
  if method not in['average','gausian_blur','median_blur','bilateral_blur']:
    raise ValueError('Please input correct method. Valid inputs:''average','gausian_blur','median_blur','bilaterial_blur')
  # if user inputs single image, check for only image formats and pass for further processing.
  if is_image== True:
    image = cv2.imread(datapath)
    if (datapath.endswith('.jpeg') or datapath.endswith('.png') or datapath.endswith('.jpg')): pass
    else: raise ValueError('please provide a valid image path for is_image=True')
    #image passes through the mentioned method for smoothing 
    #save_image= True will save the image file to the woring directory to view 
    if method== 'average':
      kernel = np.ones((5,5),np.float32)/25
      blurr = cv2.filter2D(image,-1,kernel)
      print('########### {}'.format(blurr))
      if save_image==True:
        cv2.imwrite('image_average.jpg', blurr)
        print('saved')
      smoothing= cv2.imshow('averaging',blurr) if show_image==True else blurr 

    elif method == 'gausian_blur':
      gblur = cv2.GaussianBlur(image,(5,5),0)
      if save_image==True:
        cv2.imwrite('image_gausian_blur.jpg', gblur)
      smoothing =cv2.imshow('gausian blur',gblur)  if show_image==True else gblur

    elif method == 'median_blur':
      mblurr =cv2.medianBlur(image,5)
      if save_image==True:
        cv2.imwrite('image_median_blur.jpg', mblurr)
      smoothing =cv2.imshow('median blur',mblurr)  if show_image==True else mblurr

    elif method== 'bilateral_blur':
      bilateral = cv2.bilateralFilter(image,9,75,75)
      if save_image==True:
        cv2.imwrite('image_bilateral_blur.jpg', bilateral)
      smoothing =cv2.imshow('bilateral blur',bilateral)  if show_image==True else bilateral
  #if is_image== False, loop continuous to process and check for only image format files for further processing
  elif is_image== False:
    if (datapath.endswith('.jpeg') or datapath.endswith('.png') or datapath.endswith('.jpg')): raise ValueError('please provide a valid folder path of images for is_image=False')
    else: pass
    #extract_images function gets activated and images are read and stored into image_list
    images_list = extract_images(datapath)
    #creating new list to append all the processed content into rotated_list=[]
    smoothing = []
    image_count = 1
    #image_count= 1 to intialize to 1 which will get incrimented everytime when a new data enters the list.
    for i in images_list:
      image = cv2.imread(i)
      #image passes through the mentioned method for smoothing
      if method== 'average':
        kernel = np.ones((5,5),np.float32)/25
        blurr = cv2.filter2D(image,-1,kernel)
        if save_image==True:
          cv2.imwrite('image_{}_average.jpg'.format(image_count), blurr)
          print('saved')
        smoothing_image= cv2.imshow('averaging',blurr) if show_image==True else blurr 
        smoothing.append(smoothing_image)

      elif method == 'gausian_blur':                      
        gblur = cv2.GaussianBlur(image,(5,5),0)
        if save_image==True:
          cv2.imwrite('image_{}_gausian_blur.jpg'.format(image_count), gblur)
          print('saved')
        smoothing_image =cv2.imshow('gausian blur',gblur)  if show_image==True else gblur
        smoothing.append(smoothing_image)

      elif method == 'median_blur': 
        mblurr =cv2.medianBlur(image,5)
        if save_image==True:
          cv2.imwrite('image_{}_median_blur.jpg'.format(image_count), mblurr)
          print('saved') 
        smoothing_image =cv2.imshow('median blur',mblurr)  if show_image==True else mblurr
        smoothing.append(smoothing_image)

      elif method == 'bilateral_blur':                            
        bilateral = cv2.bilateralFilter(image,9,75,75)
        if save_image==True:
          cv2.imwrite('image_{}_bilateral_blur'.format(image_count),bilateral)
          print('saved')
        smoothing_image =cv2.imshow('bilateral blur',bilateral)  if show_image==True else bilateral
        smoothing.append(smoothing_image)
       #image_count gets incrimented everytime a new file gets processed. 
      image_count+=1  
      #retrning the appended list of processed image data
  return smoothing
#%%
#image_smoothing_opencv('C:\\Users\\kavya\\Desktop\\Kavya\\preprocessing_wrappers\\scikit_opencv\\image_data\\elep.jpg',True,'average',True,False)
#%%
# Morphology
def image_morphology_opencv(datapath,is_image,method='erosion', show_image= False,save_image=False):
  # if user selects a invalid method we raise an error
  if method not in['erosion', 'dilation']:
    raise ValueError('Please input correct Operation. Valid inputs:' 'erosion,''dilation')
  # if user inputs single image, check for only image formats and pass for further processing
  if is_image==True:
    image = cv2.imread(datapath)
    if (datapath.endswith('.jpeg') or datapath.endswith('.png') or datapath.endswith('.jpg')): pass
    else: raise ValueError('please provide a valid image path for is_image=True')
    #image passes through the mentioned method for morphology
    #save_image= True will save the image file to the woring directory to view 
    if method== 'erosion':
      kernel = np.ones((5,5),np.uint8)
      erosion1 = cv2.erode(image,kernel,iterations = 1)
      if save_image==True:
        cv2.imwrite('erosion_image.jpg', erosion1)
      morphology = cv2.imshow('erosion',erosion1) if show_image==True else erosion1
    elif method == 'dilation':
      kernel = np.ones((5,5),np.uint8)
      dilation = cv2.dilate(image,kernel,iterations = 1)
      if save_image==True:
        cv2.imwrite('dilation_image.jpg', dilation)
      morphology = cv2.imshow('dilation',dilation) if show_image==True else dilation
  #if is_image== False, loop continuous to process and check for only image format files for further processing
  elif is_image== False:
    if (datapath.endswith('.jpeg') or datapath.endswith('.png') or datapath.endswith('.jpg')): raise ValueError('please provide a valid folder path of images for is_image=False')
    else: pass
    #extract_images function gets activated and images are read and stored into image_path_list
    images_list = extract_images(datapath)
    #creating new list to append all the processed content into rotated_list=[]
    morphology = []
    #image_count= 1 to intialize to 1 which will get incrimented everytime when a new data enters the list.
    image_count = 1
    for i in images_list:
      image = cv2.imread(i)
      #image passes through the mentioned method for transformation 
      if method== 'erosion':
        kernel = np.ones((5,5),np.uint8)
        erosion1 = cv2.erode(image,kernel,iterations = 1)
        if save_image==True:
          cv2.imwrite('erosion_{}_image.jpg'.format(image_count), erosion1)
        morphology_image = cv2.imshow('erosion',erosion1)if show_image==True else erosion1
        morphology.append(morphology_image)
      elif method == 'dilation':
        kernel = np.ones((5,5),np.uint8)
        dilation = cv2.dilate(image,kernel,iterations = 1)
        if save_image==True:
          cv2.imwrite('dilation_{}_image.jpg'.format(image_count), dilation)
        morphology_image = cv2.imshow('dilation',dilation) if show_image==True else dilation
        morphology.append(morphology_image)
      #image_count gets incrimented everytime a new file gets processed.
      image_count+=1
  #retrning the appended list of processed image data
  return morphology
#%%
#image_morphology_opencv('C:\\Users\\kavya\\Desktop\\Kavya\\preprocessing_wrappers\\scikit_opencv\\image_data\\elep.jpg',True,'erosion',False,False)
#%%
# Transformation
# Transformation
def image_transformation_opencv(datapath, is_image, method, show_image=False,save_image=False):
  # if user selects a invalid method we raise an error
  if method not in['rotate_anticlockwise_90', 'rotate_clockwise_90','rotate_180']:
    raise ValueError('Please input correct Operation. Valid inputs:''rotate_anticlockwise_90', 'rotate_clockwise_90','rotate_180')
  if save_image not in [True,False]:
      save_image=False  
  # if user inputs single image, check for only image formats and pass for further processing
  if is_image== True:
    image = cv2.imread(datapath)
    if (datapath.endswith('.jpeg') or datapath.endswith('.png') or datapath.endswith('.jpg')): pass
    else: raise ValueError('please provide a valid image path for is_image=True')
    #image passes through the mentioned method for transformation
    #save_image= True will save the image file to the woring directory to view  
    if method == 'rotate_clockwise_90':
      img_rotate_90_clockwise = cv2.rotate(image, cv2.ROTATE_90_CLOCKWISE)
      if save_image==True:
        cv2.imwrite('image_rotate_90_clockwise.jpg', img_rotate_90_clockwise)
      transformation = cv2.imshow('rotated_90_clockwise',img_rotate_90_clockwise) if show_image==True else img_rotate_90_clockwise

    elif method == 'rotate_anticlockwise_90':
      img_rotate_90_counterclockwise = cv2.rotate(image, cv2.ROTATE_90_COUNTERCLOCKWISE)
      if save_image==True:
        cv2.imwrite('image_rotate_90_counterclockwise.jpg', img_rotate_90_counterclockwise)
      transformation =cv2.imshow('rotated_90_counter_clockwise',img_rotate_90_counterclockwise) if show_image==True else img_rotate_90_counterclockwise

    elif method == 'rotate_180':
      img_rotate_180 = cv2.rotate(image, cv2.ROTATE_180)
      if save_image==True:
        cv2.imwrite('image_rotate_180.jpg', img_rotate_180)
      transformation= cv2.imshow('rotated_180',img_rotate_180) if show_image==True else img_rotate_180 
  #if is_image== False, loop continuous to process and check for only image format files for further processing   
  elif is_image== False:

    if (datapath.endswith('.jpeg') or datapath.endswith('.png') or datapath.endswith('.jpg')): raise ValueError('please provide a valid folder path of images for is_image=False')
    else: pass
    #extract_images function gets activated and images are read and stored into image_path_list
    images_list = extract_images(datapath)
    #creating new list to append all the processed content into transformation=[]
    transformation= []
    #image_count= 1 to intialize to 1 which will get incrimented everytime when a new data enters the list.
    image_count = 1
    for i in images_list:
      image = cv2.imread(i)
      #image passes through the mentioned method for transformation 
      if method == 'rotate_clockwise_90':
        img_rotate_90_clockwise = cv2.rotate(image, cv2.ROTATE_90_CLOCKWISE)
        if save_image==True:
          cv2.imwrite('image_{}_rotate_90_clockwise.jpg'.format(image_count), img_rotate_90_clockwise)
        rotate90 = cv2.imshow('rotated_90_clockwise',img_rotate_90_clockwise) if show_image==True else img_rotate_90_clockwise
        transformation.append(rotate90)

      elif method== 'rotate_anticlockwise_90':
        img_rotate_90_counterclockwise = cv2.rotate(image, cv2.ROTATE_90_COUNTERCLOCKWISE)
        if save_image==True:
          cv2.imwrite('image_{}_rotate_90_counterclockwise.jpg'.format(image_count), img_rotate_90_counterclockwise)
        rotate90c =cv2.imshow('rotated_90_counter_clockwise',img_rotate_90_counterclockwise) if show_image==True else img_rotate_90_counterclockwise
        transformation.append(rotate90c)
      elif method == 'rotate_180':
        img_rotate_180 = cv2.rotate(image, cv2.ROTATE_180)
        if save_image==True:
          cv2.imwrite('image_{}_rotate_180.jpg'.format(image_count), img_rotate_180)
          print('saved')
        rotate90ac= cv2.imshow('rotated_180',img_rotate_180) if show_image==True else img_rotate_180 
        transformation.append(rotate90ac)
      #image_count gets incrimented everytime a new file gets processed.
      image_count += 1
  #retrning the appended list of processed image data
  return transformation
#%%
#image_transformation_opencv('C:\\Users\\kavya\\Desktop\\Kavya\\preprocessing_wrappers\\scikit_opencv\\image_data\\elep.jpg',True,'rotate_180',False,False)
#%%
def image_colorspace_opencv(datapath, is_image, method='grey', show_image= False,save_image=False):
    # if user selects a invalid method we raise an error
    if method not in['green', 'grey','hsv','blue']:
        raise ValueError('Please input correct method. Valid inputs:''green', 'grey','hsv','blue')
    # if user inputs single image, check for only image formats and pass for further processing
    if is_image== True:
        image = cv2.imread(datapath)
        if (datapath.endswith('.jpeg') or datapath.endswith('.png') or datapath.endswith('.jpg')): pass
        else: raise ValueError('please provide a valid image path for is_image=True')
        #image passes through the mentioned method for colorspace
        #save_image= True will save the image file to the woring directory to view 
        if method == 'green':
            green = cv2.cvtColor(image, cv2.COLOR_BGR2RGBA)
            if save_image==True:
                cv2.imwrite('image_green.jpg', green)
                print('saved')
            colorspace=cv2.imshow('green',green) if show_image==True else green
            cv2.waitKey(0)
            cv2.destroyAllWindows() 

        elif method == 'grey':
            bgrgray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
            if save_image==True:
                cv2.imwrite('image_grey.jpg', bgrgray)
                print('saved')      
            colorspace=cv2.imshow('grey',bgrgray) if show_image==True else bgrgray
            cv2.waitKey(0)
            cv2.destroyAllWindows() 

        elif method == 'hsv':
            bgrhsv = cv2.cvtColor(image, cv2.COLOR_BGR2HSV)
            if save_image==True:
                cv2.imwrite('image_hsv.jpg', bgrhsv)
                print('saved')
            colorspace=cv2.imshow('hsv',bgrhsv) if show_image==True else bgrhsv
            cv2.waitKey(0)
            cv2.destroyAllWindows() 

        elif method == 'blue':
            bluei= cv2.cvtColor(image,cv2.COLOR_Lab2BGR)
            if save_image==True:
                cv2.imwrite('image_blue.jpg', bluei)
                print('saved')
            colorspace=cv2.imshow('blue',bluei) if show_image==True else bluei
            cv2.waitKey(0)
            cv2.destroyAllWindows() 
    #if is_image== False, loop continuous to process and check for only image format files for further processing
    elif is_image== False:

        if (datapath.endswith('.jpeg') or datapath.endswith('.png') or datapath.endswith('.jpg')): raise ValueError('please provide a valid folder path of images for is_image=False')
        else: pass
        #extract_images function gets activated and images are read and stored into image_path_list
        images_list = extract_images(datapath)
        #creating new list to append all the processed content into rotated_list=[]
        colorspace= []
        #image_count= 1 to intialize to 1 which will get incrimented everytime when a new data enters the list.
        image_count = 1
        for i in images_list:
            image = cv2.imread(i)
            #image passes through the mentioned method for colorspace
            if method == 'green':
                green = cv2.cvtColor(image, cv2.COLOR_BGR2RGBA)
                print('##################### {}'.format(green))
                if save_image==True:
                    cv2.imwrite('image_{}_green.jpg'.format(image_count), green)
                print('saved')        
                colorspaceg=cv2.imshow('green',green) if show_image==True else green
                cv2.waitKey(0)
                cv2.destroyAllWindows() 
                colorspace.append(colorspaceg)

            elif method == 'grey':
                bgrgray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
                if save_image==True:
                    cv2.imwrite('image_{}_grey.jpg'.format(image_count), bgrgray)
                print('saved')
                colorspacegr=cv2.imshow('grey',bgrgray) if show_image==True else bgrgray
                cv2.waitKey(0)
                cv2.destroyAllWindows() 
                colorspace.append(colorspacegr)

            elif method == 'hsv':

                bgrhsv = cv2.cvtColor(image, cv2.COLOR_BGR2HSV)
                if save_image==True:
                    cv2.imwrite('image_{}_hsv.jpg'.format(image_count), bgrhsv)
                print('saved')
                colorspacehsv=cv2.imshow('hsv',bgrhsv) if show_image==True else bgrhsv
                cv2.waitKey(0)
                cv2.destroyAllWindows() 
                colorspace.append(colorspacehsv)
        
            elif method == "blue":              
                bluei= cv2.cvtColor(image,cv2.COLOR_Lab2BGR)
                if save_image==True:
                    cv2.imwrite('image_{}_blue.jpg'.format(image_count), bluei)
                print('saved')
                colorspaceb=cv2.imshow('blue',bluei) if show_image==True else bluei
                cv2.waitKey(0)
                cv2.destroyAllWindows() 
                colorspace.append(colorspaceb)
            #image_count gets incrimented everytime a new file gets processed.
            image_count+=1
    #retrning the appended list of processed image data
    return colorspace
#%%
#image_colorspace_opencv('C:\\Users\\kavya\\Desktop\\Kavya\\preprocessing_wrappers\\scikit_opencv\\image_data\\elep.jpg',True,'grey',False,False)

#%%  
##segmentation
##segmentation
def image_segmentation_opencv(datapath,is_image,show_image= False,save_image=False):
    # if user inputs single image, check for only image formats and pass for further processing
    #image passes through the mentioned method for segmentation 
    #save_image= True will save the image file to the woring directory to view 
    if is_image== True:
        image = cv2.imread(datapath)
        if (datapath.endswith('.jpeg') or datapath.endswith('.png') or datapath.endswith('.jpg')): pass
        else: raise ValueError('please provide a valid image path for is_image=True')
    #input image gets converted to grayscale image for processing of segmentation
        gray = cv2.cvtColor(image,cv2.COLOR_BGR2GRAY)
        #cv2.threshold parameters are 
        ret, thresh = cv2.threshold(gray,0,255,cv2.THRESH_BINARY_INV+cv2.THRESH_OTSU)
        kernel = np.ones((3,3),np.uint8)
        opening = cv2.morphologyEx(thresh,cv2.MORPH_OPEN,kernel, iterations = 2)
        sure_bg = cv2.dilate(opening,kernel,iterations=3)
        dist_transform = cv2.distanceTransform(opening,cv2.DIST_L2,5)
        ret, sure_fg = cv2.threshold(dist_transform,0.7*dist_transform.max(),255,0)
        sure_fg = np.uint8(sure_fg)
        unknown = cv2.subtract(sure_bg,sure_fg)
        ret, markers = cv2.connectedComponents(sure_fg)
        markers = markers+1
        markers[unknown==255] = 0
        markers = cv2.watershed(image,markers)
        image[markers == -1] = [255,0,0]
        if save_image==True:
            cv2.imwrite('segmentation_image.jpg', image)
        segmentation= cv2.imshow('segmentation',image) if show_image==True else image
    #if is_image== False, loop continuous to process and check for only image format files for further processing.
    elif is_image== False:
        if (datapath.endswith('.jpeg') or datapath.endswith('.png') or datapath.endswith('.jpg')): raise ValueError('please provide a valid folder path of images for is_image=False')
        else: pass
        images_list = extract_images(datapath)
        segmentation= []
        image_count = 1
        for i in images_list:
            image = cv2.imread(i)
            #input image gets converted to grayscale image for processing of segmentation
            gray = cv2.cvtColor(image,cv2.COLOR_BGR2GRAY)
            #threshold is applied to smooth the image
            ret, thresh = cv2.threshold(gray,0,255,cv2.THRESH_BINARY_INV+cv2.THRESH_OTSU)
            kernel = np.ones((3,3),np.uint8)
            #To use the openig operation, we’ll use the morphologyEx function from the OpenCV Python bindings.
            opening = cv2.morphologyEx(thresh,cv2.MORPH_OPEN,kernel, iterations = 2)   
            #extracting sure background using morphology dilation from the OpenCV Python bindings  
            sure_bg = cv2.dilate(opening,kernel,iterations=3)
            #calculating distance with distaceTransform from the OpenCV Python bindings
            dist_transform = cv2.distanceTransform(opening,cv2.DIST_L2,5)
            ret, sure_fg = cv2.threshold(dist_transform,0.7*dist_transform.max(),255,0)
            #extracting sure foreground
            sure_fg = np.uint8(sure_fg)
            #unknown/ unidentified area from the picture is produced by subtracting fore
            unknown = cv2.subtract(sure_bg,sure_fg)
            ret, markers = cv2.connectedComponents(sure_fg)
            markers = markers+1
            markers[unknown==255] = 0
            markers = cv2.watershed(image,markers)
            image[markers == -1] = [255,0,0]
            segmimg= cv2.imshow('segmentation',image) if show_image==True else image
            if save_image==True:
                cv2.imwrite('segmentation_image_{}.jpg'.format(image_count), image)
            segmentation.append(segmimg) 
            #image_count gets incrimented everytime a new file gets processed.
            image_count+=1
    #retrning the appended list of processed image data
    return segmentation
#%%
#image_segmentation_opencv('C:\\Users\\kavya\\Desktop\\Kavya\\preprocessing_wrappers\\scikit_opencv\\image_data\\elep.jpg',True,False,False)
#%%
#RESIZE
#RESIZE
def image_resize_opencv(datapath ,is_image ,method='resize-big',show_image=False, save_image=False, width = None, height = None):
    # if user selects a invalid method we raise an error
    if method not in ['custom', 'resize-small','resize-big','resize-stretch-horizontal','resize-stretch-vertical']:
        raise ValueError('Please input correct method. Valid inputs:''custom', 'resize_small','resize_big','resize-stretch-horizontal','resize-stretch-vertical')
    image = cv2.imread(datapath)
    #print(image)
    # if user inputs single image, check for only image formats and pass for further processing
    if is_image==True:
        if (datapath.endswith('.jpeg') or datapath.endswith('.png') or datapath.endswith('.jpg')): pass
        else: raise ValueError('please provide a valid image path for is_image=True')
        #image passes through the mentioned method for resize
        #save_image= True will save the image file to the woring directory to view 
        if method=='custom':
            if width & height == None:
                raise ValueError("please width and height")
            else:
                print('Original Dimensions : ' ,image.shape)
                dim = (width, height)
                resized = cv2.resize(image, dim, interpolation = cv2.INTER_AREA)
                if save_image==True:
                    cv2.imwrite('image_custom_resize.jpg', resized)
                print('Resized Dimensions : ' ,resized.shape)
                #cv2.imshow(image)
                resizec =cv2.imshow('resized',resized) if show_image==True else resized
                cv2.waitKey(0)  
                cv2.destroyAllWindows() 

        elif method == 'resize-small':
            img = image
            print('Original Dimensions : ' ,img.shape)
            scale_percent = 60 # percent of original size
            width = int(img.shape[1] * scale_percent / 100)
            height = int(img.shape[0] * scale_percent / 100)
            dim = (width, height)
            resized = cv2.resize(img, dim, interpolation = cv2.INTER_AREA)
            if save_image==True:
                cv2.imwrite('image_resize_small.jpg', resized)
            print('Resized Dimensions : ' ,resized.shape)
            resize= cv2.imshow('resized',resized) if show_image==True else resized
            cv2.waitKey(0)
            cv2.destroyAllWindows() 

        elif method == 'resize-big':
            img = image
            print('Original Dimensions : ' ,img.shape)
            scale_percent = 220 # percent of original size
            width = int(img.shape[1] * scale_percent / 100)
            height = int(img.shape[0] * scale_percent / 100)
            dim = (width, height)
            resized = cv2.resize(img, dim, interpolation = cv2.INTER_AREA)
            if save_image==True:
                cv2.imwrite('image_resize_big.jpg', resized)
            print('Resized Dimensions : ' ,resized.shape)
            resize =cv2.imshow('resized',resized) if show_image==True else resized
            cv2.waitKey(0)
            cv2.destroyAllWindows() 

        elif method == 'resize-stretch-horizontal':
            img = image
            print('Original Dimensions : ',img.shape)
            width = 440
            height = img.shape[0] # keep original height
            dim = (width, height)
            resized = cv2.resize(img, dim, interpolation = cv2.INTER_AREA)
            if save_image==True:
                cv2.imwrite('image_resize_stretch-horizontal.jpg', resized)
            print('Resized Dimensions : ' ,resized.shape)
            resize =cv2.imshow('resized',resized) if show_image==True else resized
            cv2.waitKey(0)
            cv2.destroyAllWindows() 

        elif method== 'resize-stretch-vertical':
            print('entered')
            img= image 
            width = img.shape[1] # keep original width
            height = 440
            dim = (width, height)
            resized = cv2.resize(img, dim, interpolation = cv2.INTER_AREA)
            print('resized')
            if save_image==True:
                cv2.imwrite('image_resize_stretch-vertical.jpg', resized)
            print('saved')
            resize= cv2.imshow('resized',resized) if show_image==True else resized
            print('resize saved')
            cv2.waitKey(0)
            cv2.destroyAllWindows() 
    #if is_image== False, loop continuous to process and check for only image format files for further processing
    elif is_image== False:
        if (datapath.endswith('.jpeg') or datapath.endswith('.png') or datapath.endswith('.jpg')): raise ValueError('please provide a valid folder path of images for is_image=False')
        else: pass
        #extract_images function gets activated and images are read and stored into image_list
        images_list = extract_images(datapath)
        #creating new list to append all the processed content into rotated_list=[]
        resize = []
        image_count= 1
        #image_count= 1 to intialize to 1 which will get incrimented everytime when a new data enters the list.
        for i in images_list:
            image = cv2.imread(i)
            #image passes through the mentioned method for resize
            if method== 'custom':
                if width & height == None:
                    raise ValueError("please width and height")
                else:
                    print('Original Dimensions : ' ,image.shape)
                    dim = (width, height)
                    resized = cv2.resize(image, dim, interpolation = cv2.INTER_AREA)
                    if save_image==True:
                        cv2.imwrite('image_{}_resize_custom.jpg'.format(image_count), resized)
                        print('saved')
                    print('Resized Dimensions : ' ,resized.shape)
                    cv2.imshow('',image)
                    resizec =cv2.imshow('resized',resized) if show_image==True else resized
                    cv2.waitKey(0)
                    cv2.destroyAllWindows() 
                    resize.append(resizec)

            elif method== 'resize-small':
                img = image
                print('Original Dimensions : ' ,img.shape)
                scale_percent = 60 # percent of original size
                width = int(img.shape[1] * scale_percent / 100)
                height = int(img.shape[0] * scale_percent / 100)
                dim = (width, height)
                resized = cv2.resize(img, dim, interpolation = cv2.INTER_AREA)
                if save_image==True:
                    cv2.imwrite('image_{}_resize_small.jpg'.format(image_count), resized)
                    print('saved')
                    #print('Resized Dimensions : ' ,resized.shape)
                cv2.imshow('',img)
                resizes= cv2.imshow('resized',resized) if show_image==True else resized
                cv2.waitKey(0)
                cv2.destroyAllWindows() 
                resize.append(resizes)

            elif method == 'resize-big':
                img = image
                print('Original Dimensions : ' ,img.shape)
                scale_percent = 220
                width = int(img.shape[1] * scale_percent / 100)
                height = int(img.shape[0] * scale_percent / 100)
                dim = (width, height)
                resized = cv2.resize(img, dim, interpolation = cv2.INTER_AREA)
                if save_image==True:
                    cv2.imwrite('image_{}_resize_big.jpg'.format(image_count), resized)
                print('saved')
                print('Resized Dimensions : ' ,resized.shape)
                cv2.imshow('',img)
                resizeb =cv2.imshow('resized',resized) if show_image==True else resized
                cv2.waitKey(0)
                cv2.destroyAllWindows() 
                resize.append(resizeb)

            elif method == 'resize-stretch-horizontal':
                img = image
                print('Original Dimensions : ' ,img.shape)
                width = 440
                height = img.shape[0] 
                dim = (width, height)
                resized = cv2.resize(img, dim, interpolation = cv2.INTER_AREA)
                if save_image==True:
                    cv2.imwrite('image_{}_resize-stretch-horizontal.jpg'.format(image_count), resized)
                print('saved')
                print('Resized Dimensions : ' ,resized.shape)
                cv2.imshow('',img)
                resizesh =cv2.imshow('resized',resized) if show_image==True else resized
                cv2.waitKey(0)
                cv2.destroyAllWindows() 
                resize.append(resizesh)
            elif method== 'resize-stretch-vertical':
                img = image 
                width = img.shape[1] 
                height = 440
                dim = (width, height)
                resized = cv2.resize(img, dim, interpolation = cv2.INTER_AREA)
                if save_image==True:
                    cv2.imwrite('image_{}_resize-stretch-vertical.jpg'.format(image_count), resized)
                print('saved')
                cv2.imshow('', img)
                resizesv= cv2.imshow('resized',resized) if show_image==True else resized
                cv2.waitKey(0)
                cv2.destroyAllWindows() 
                resize.append(resizesv)
            #image_count gets incrimented everytime a new file gets processed.
            image_count+=1
    #retrning the appended list of processed image data   
    return resize


# %%
#image_resize_opencv('C:\\Users\\kavya\\Desktop\\Kavya\\preprocessing_wrappers\\scikit_opencv\\image_data\\elep.jpg',True,'resize-small',show_image=True,save_image=True)

# %%



# -*- coding: utf-8 -*-
"""
Created on Sat Oct 17 17:57:50 2020

@author: kavya
"""

import pandas as pd
import numpy as np
import copy

def create_dataframe():
    return(pd.DataFrame())

def read_pickle(filepath_or_buffer, compression='infer'):
    return(pd.read_pickle(filepath_or_buffer, compression=compression))
    
def read_table(filepath_or_buffer, sep='\t', delimiter=None, header='infer', names=None, index_col=None, usecols=None, squeeze=False, prefix=None, mangle_dupe_cols=True, dtype=None, engine=None, converters=None, true_values=None, false_values=None, skipinitialspace=False, skiprows=None, skipfooter=0, nrows=None, na_values=None, keep_default_na=True, na_filter=True, verbose=False, skip_blank_lines=True, parse_dates=False, infer_datetime_format=False, keep_date_col=False, date_parser=None, dayfirst=False, cache_dates=True, iterator=False, chunksize=None, compression='infer', thousands=None, decimal='.', lineterminator=None, quotechar='"', quoting=0, doublequote=True, escapechar=None, comment=None, encoding=None, dialect=None, error_bad_lines=True, warn_bad_lines=True, delim_whitespace=False, low_memory=True, memory_map=False, float_precision=None):
    return(pd.read_table(filepath_or_buffer=filepath_or_buffer, sep=sep, delimiter=delimiter, header=header, names=names, index_col=index_col, usecols=usecols, squeeze=squeeze, prefix=prefix, mangle_dupe_cols=mangle_dupe_cols, dtype=dtype, engine=engine, converters=converters, true_values=true_values, false_values=false_values, skipinitialspace=skipinitialspace, skiprows=skiprows, skipfooter=skipfooter, nrows=nrows, na_values=na_values, keep_default_na=keep_default_na, na_filter=na_filter, verbose=verbose, skip_blank_lines=skip_blank_lines, parse_dates=parse_dates, infer_datetime_format=infer_datetime_format, keep_date_col=keep_date_col, date_parser=date_parser, dayfirst=dayfirst, cache_dates=cache_dates, iterator=iterator, chunksize=chunksize, compression=compression, thousands=thousands, decimal=decimal, lineterminator=lineterminator, quotechar=quotechar, quoting=quoting, doublequote=doublequote, escapechar=escapechar, comment=comment, encoding=encoding, dialect=dialect, error_bad_lines=error_bad_lines, warn_bad_lines=warn_bad_lines, delim_whitespace=delim_whitespace, low_memory=low_memory, memory_map=memory_map, float_precision=float_precision))


def read_csv(filepath_or_buffer, sep=',', delimiter=None, header='infer', names=None, index_col=None, usecols=None, squeeze=False, prefix=None, mangle_dupe_cols=True, dtype=None, engine=None, converters=None, true_values=None, false_values=None, skipinitialspace=False, skiprows=None, skipfooter=0, nrows=None, na_values=None, keep_default_na=True, na_filter=True, verbose=False, skip_blank_lines=True, parse_dates=False, infer_datetime_format=False, keep_date_col=False, date_parser=None, dayfirst=False, cache_dates=True, iterator=False, chunksize=None, compression='infer', thousands=None, decimal='.', lineterminator=None, quotechar='"', quoting=0, doublequote=True, escapechar=None, comment=None, encoding=None, dialect=None, error_bad_lines=True, warn_bad_lines=True, delim_whitespace=False, low_memory=True, memory_map=False, float_precision=None):
    """
    Parameters
filepath_or_bufferstr, path object or file-like object
Any valid string path is acceptable. The string could be a URL. Valid URL schemes include http, ftp, s3, gs, and file. For file URLs, a host is expected. A local file could be: file://localhost/path/to/table.csv.

If you want to pass in a path object, preprocessor accepts any os.PathLike.

By file-like object, we refer to objects with a read() method, such as a file handler (e.g. via builtin open function) or StringIO.

sepstr, default ‘,’
Delimiter to use. If sep is None, the C engine cannot automatically detect the separator, but the Python parsing engine can, meaning the latter will be used and automatically detect the separator by Python’s builtin sniffer tool, csv.Sniffer. In addition, separators longer than 1 character and different from '\s+' will be interpreted as regular expressions and will also force the use of the Python parsing engine. Note that regex delimiters are prone to ignoring quoted data. Regex example: '\r\t'.

delimiterstr, default None
Alias for sep.

headerint, list of int, default ‘infer’
Row number(s) to use as the column names, and the start of the data. Default behavior is to infer the column names: if no names are passed the behavior is identical to header=0 and column names are inferred from the first line of the file, if column names are passed explicitly then the behavior is identical to header=None. Explicitly pass header=0 to be able to replace existing names. The header can be a list of integers that specify row locations for a multi-index on the columns e.g. [0,1,3]. Intervening rows that are not specified will be skipped (e.g. 2 in this example is skipped). Note that this parameter ignores commented lines and empty lines if skip_blank_lines=True, so header=0 denotes the first line of data rather than the first line of the file.

namesarray-like, optional
List of column names to use. If the file contains a header row, then you should explicitly pass header=0 to override the column names. Duplicates in this list are not allowed.

index_colint, str, sequence of int / str, or False, default None
Column(s) to use as the row labels of the DataFrame, either given as string name or column index. If a sequence of int / str is given, a MultiIndex is used.

Note: index_col=False can be used to force preprocessor to not use the first column as the index, e.g. when you have a malformed file with delimiters at the end of each line.

usecolslist-like or callable, optional
Return a subset of the columns. If list-like, all elements must either be positional (i.e. integer indices into the document columns) or strings that correspond to column names provided either by the user in names or inferred from the document header row(s). For example, a valid list-like usecols parameter would be [0, 1, 2] or ['foo', 'bar', 'baz']. Element order is ignored, so usecols=[0, 1] is the same as [1, 0]. To instantiate a DataFrame from data with element order preserved use pd.read_csv(data, usecols=['foo', 'bar'])[['foo', 'bar']] for columns in ['foo', 'bar'] order or pd.read_csv(data, usecols=['foo', 'bar'])[['bar', 'foo']] for ['bar', 'foo'] order.

If callable, the callable function will be evaluated against the column names, returning names where the callable function evaluates to True. An example of a valid callable argument would be lambda x: x.upper() in ['AAA', 'BBB', 'DDD']. Using this parameter results in much faster parsing time and lower memory usage.

squeezebool, default False
If the parsed data only contains one column then return a Series.

prefixstr, optional
Prefix to add to column numbers when no header, e.g. ‘X’ for X0, X1, …

mangle_dupe_colsbool, default True
Duplicate columns will be specified as ‘X’, ‘X.1’, …’X.N’, rather than ‘X’…’X’. Passing in False will cause data to be overwritten if there are duplicate names in the columns.

dtypeType name or dict of column -> type, optional
Data type for data or columns. E.g. {‘a’: np.float64, ‘b’: np.int32, ‘c’: ‘Int64’} Use str or object together with suitable na_values settings to preserve and not interpret dtype. If converters are specified, they will be applied INSTEAD of dtype conversion.

engine{‘c’, ‘python’}, optional
Parser engine to use. The C engine is faster while the python engine is currently more feature-complete.

convertersdict, optional
Dict of functions for converting values in certain columns. Keys can either be integers or column labels.

true_valueslist, optional
Values to consider as True.

false_valueslist, optional
Values to consider as False.

skipinitialspacebool, default False
Skip spaces after delimiter.

skiprowslist-like, int or callable, optional
Line numbers to skip (0-indexed) or number of lines to skip (int) at the start of the file.

If callable, the callable function will be evaluated against the row indices, returning True if the row should be skipped and False otherwise. An example of a valid callable argument would be lambda x: x in [0, 2].

skipfooterint, default 0
Number of lines at bottom of file to skip (Unsupported with engine=’c’).

nrowsint, optional
Number of rows of file to read. Useful for reading pieces of large files.

na_valuesscalar, str, list-like, or dict, optional
Additional strings to recognize as NA/NaN. If dict passed, specific per-column NA values. By default the following values are interpreted as NaN: ‘’, ‘#N/A’, ‘#N/A N/A’, ‘#NA’, ‘-1.#IND’, ‘-1.#QNAN’, ‘-NaN’, ‘-nan’, ‘1.#IND’, ‘1.#QNAN’, ‘<NA>’, ‘N/A’, ‘NA’, ‘NULL’, ‘NaN’, ‘n/a’, ‘nan’, ‘null’.

keep_default_nabool, default True
Whether or not to include the default NaN values when parsing the data. Depending on whether na_values is passed in, the behavior is as follows:

If keep_default_na is True, and na_values are specified, na_values is appended to the default NaN values used for parsing.

If keep_default_na is True, and na_values are not specified, only the default NaN values are used for parsing.

If keep_default_na is False, and na_values are specified, only the NaN values specified na_values are used for parsing.

If keep_default_na is False, and na_values are not specified, no strings will be parsed as NaN.

Note that if na_filter is passed in as False, the keep_default_na and na_values parameters will be ignored.

na_filterbool, default True
Detect missing value markers (empty strings and the value of na_values). In data without any NAs, passing na_filter=False can improve the performance of reading a large file.

verbosebool, default False
Indicate number of NA values placed in non-numeric columns.

skip_blank_linesbool, default True
If True, skip over blank lines rather than interpreting as NaN values.

parse_datesbool or list of int or names or list of lists or dict, default False
The behavior is as follows:

boolean. If True -> try parsing the index.

list of int or names. e.g. If [1, 2, 3] -> try parsing columns 1, 2, 3 each as a separate date column.

list of lists. e.g. If [[1, 3]] -> combine columns 1 and 3 and parse as a single date column.

dict, e.g. {‘foo’ : [1, 3]} -> parse columns 1, 3 as date and call result ‘foo’

If a column or index cannot be represented as an array of datetimes, say because of an unparseable value or a mixture of timezones, the column or index will be returned unaltered as an object data type. For non-standard datetime parsing, use pd.to_datetime after pd.read_csv. To parse an index or column with a mixture of timezones, specify date_parser to be a partially-applied pandas.to_datetime() with utc=True. See Parsing a CSV with mixed timezones for more.

Note: A fast-path exists for iso8601-formatted dates.

infer_datetime_formatbool, default False
If True and parse_dates is enabled, preprocessor will attempt to infer the format of the datetime strings in the columns, and if it can be inferred, switch to a faster method of parsing them. In some cases this can increase the parsing speed by 5-10x.

keep_date_colbool, default False
If True and parse_dates specifies combining multiple columns then keep the original columns.

date_parserfunction, optional
Function to use for converting a sequence of string columns to an array of datetime instances. The default uses dateutil.parser.parser to do the conversion. Pandas will try to call date_parser in three different ways, advancing to the next if an exception occurs: 1) Pass one or more arrays (as defined by parse_dates) as arguments; 2) concatenate (row-wise) the string values from the columns defined by parse_dates into a single array and pass that; and 3) call date_parser once for each row using one or more strings (corresponding to the columns defined by parse_dates) as arguments.

dayfirstbool, default False
DD/MM format dates, international and European format.

cache_datesbool, default True
If True, use a cache of unique, converted dates to apply the datetime conversion. May produce significant speed-up when parsing duplicate date strings, especially ones with timezone offsets.

New in version 0.25.0.

iteratorbool, default False
Return TextFileReader object for iteration or getting chunks with get_chunk().

chunksizeint, optional
Return TextFileReader object for iteration. See the IO Tools docs for more information on iterator and chunksize.

compression{‘infer’, ‘gzip’, ‘bz2’, ‘zip’, ‘xz’, None}, default ‘infer’
For on-the-fly decompression of on-disk data. If ‘infer’ and filepath_or_buffer is path-like, then detect compression from the following extensions: ‘.gz’, ‘.bz2’, ‘.zip’, or ‘.xz’ (otherwise no decompression). If using ‘zip’, the ZIP file must contain only one data file to be read in. Set to None for no decompression.

thousandsstr, optional
Thousands separator.

decimalstr, default ‘.’
Character to recognize as decimal point (e.g. use ‘,’ for European data).

lineterminatorstr (length 1), optional
Character to break file into lines. Only valid with C parser.

quotecharstr (length 1), optional
The character used to denote the start and end of a quoted item. Quoted items can include the delimiter and it will be ignored.

quotingint or csv.QUOTE_* instance, default 0
Control field quoting behavior per csv.QUOTE_* constants. Use one of QUOTE_MINIMAL (0), QUOTE_ALL (1), QUOTE_NONNUMERIC (2) or QUOTE_NONE (3).

doublequotebool, default True
When quotechar is specified and quoting is not QUOTE_NONE, indicate whether or not to interpret two consecutive quotechar elements INSIDE a field as a single quotechar element.

escapecharstr (length 1), optional
One-character string used to escape other characters.

commentstr, optional
Indicates remainder of line should not be parsed. If found at the beginning of a line, the line will be ignored altogether. This parameter must be a single character. Like empty lines (as long as skip_blank_lines=True), fully commented lines are ignored by the parameter header but not by skiprows. For example, if comment='#', parsing #empty\na,b,c\n1,2,3 with header=0 will result in ‘a,b,c’ being treated as the header.

encodingstr, optional
Encoding to use for UTF when reading/writing (ex. ‘utf-8’). List of Python standard encodings .

dialectstr or csv.Dialect, optional
If provided, this parameter will override values (default or not) for the following parameters: delimiter, doublequote, escapechar, skipinitialspace, quotechar, and quoting. If it is necessary to override values, a ParserWarning will be issued. See csv.Dialect documentation for more details.

error_bad_linesbool, default True
Lines with too many fields (e.g. a csv line with too many commas) will by default cause an exception to be raised, and no DataFrame will be returned. If False, then these “bad lines” will dropped from the DataFrame that is returned.

warn_bad_linesbool, default True
If error_bad_lines is False, and warn_bad_lines is True, a warning for each “bad line” will be output.

delim_whitespacebool, default False
Specifies whether or not whitespace (e.g. ' ' or '    ') will be used as the sep. Equivalent to setting sep='\s+'. If this option is set to True, nothing should be passed in for the delimiter parameter.

low_memorybool, default True
Internally process the file in chunks, resulting in lower memory use while parsing, but possibly mixed type inference. To ensure no mixed types either set False, or specify the type with the dtype parameter. Note that the entire file is read into a single DataFrame regardless, use the chunksize or iterator parameter to return the data in chunks. (Only valid with C parser).

memory_mapbool, default False
If a filepath is provided for filepath_or_buffer, map the file object directly onto memory and access the data directly from there. Using this option can improve performance because there is no longer any I/O overhead.

float_precisionstr, optional
Specifies which converter the C engine should use for floating-point values. The options are None for the ordinary converter, high for the high-precision converter, and round_trip for the round-trip converter.

Returns
DataFrame or TextParser
A comma-separated values (csv) file is returned as two-dimensional data structure with labeled axes.

    """
    return(pd.read_csv(filepath_or_buffer=filepath_or_buffer, sep=sep, delimiter=delimiter, header=header, names=names, index_col=index_col, usecols=usecols, squeeze=squeeze, prefix=prefix, mangle_dupe_cols=mangle_dupe_cols, dtype=dtype, engine=engine, converters=converters, true_values=true_values, false_values=false_values, skipinitialspace=skipinitialspace, skiprows=skiprows, skipfooter=skipfooter, nrows=nrows, na_values=na_values, keep_default_na=keep_default_na, na_filter=na_filter, verbose=verbose, skip_blank_lines=skip_blank_lines, parse_dates=parse_dates, infer_datetime_format=infer_datetime_format, keep_date_col=keep_date_col, date_parser=date_parser, dayfirst=dayfirst, cache_dates=cache_dates, iterator=iterator, chunksize=chunksize, compression=compression, thousands=thousands, decimal=decimal, lineterminator=lineterminator, quotechar=quotechar, quoting=quoting, doublequote=doublequote, escapechar=escapechar, comment=comment, encoding=encoding, dialect=dialect, error_bad_lines=error_bad_lines, warn_bad_lines=warn_bad_lines, delim_whitespace=delim_whitespace, low_memory=low_memory, memory_map=memory_map, float_precision=float_precision))

#pd.read_csv('C:\\Users\\kavya\\Desktop\\Kavya\\DL_Wrapper\\data\\heart\\heart.csv')


def read_fwf(filepath_or_buffer, colspecs='infer', widths=None, infer_nrows=100, **kwds):
    return(pd.read_fwf(filepath_or_buffer=filepath_or_buffer,colspecs=colspecs,widths=widths,infer_nrows=infer_nrows,**kwds))

def read_clipboard(sep='\\s+', **kwargs):
    return(pd.read_clipboard(sep=sep,**kwargs))



def read_excel(io,sheet_name=0,header=0,names=None,index_col=None,usecols=None,squeeze=False,dtype=None,engine=None,converters=None,true_values=None,false_values=None,skiprows=None,nrows=None,na_values=None,keep_default_na=True,na_filter=True,verbose=False,parse_dates=False,date_parser=None,thousands=None,comment=None,skipfooter=0,convert_float=True,mangle_dupe_cols=True):
    return(pd.read_excel(io=io,sheet_name=sheet_name,header=header,names=names,index_col=index_col,usecols=usecols,squeeze=squeeze,dtype=dtype,engine=engine,converters=converters,date_parser=date_parser,true_values=true_values,false_values=false_values,skiprows=skiprows,nrows=nrows,na_values=na_values,keep_default_na=keep_default_na,na_filter=na_filter,verbose=verbose,parse_dates=parse_dates,thousands=thousands,comment=comment,skipfooter=skipfooter,convert_float=convert_float,mangle_dupe_cols=mangle_dupe_cols))

def ExcelFile_parse(sheet_name=0,header=0,names=None,index_col=None,usecols=None,squeeze=False,converters=None,true_values=None,false_values=None,skiprows=None,nrows=None,na_values=None,parse_dates=False,date_parser=None,thousands=None,comment=None,skipfooter=0,convert_float=True, mangle_dupe_cols=True,**kwds):
    return(pd.ExcelFile.parse(sheet_name=sheet_name,header=header,names=names,index_col=index_col,usecols=usecols,squeeze=squeeze,converters=converters,true_values=true_values,false_values=false_values,skiprows=skiprows,nrows=nrows,na_values=na_values,parse_dates=parse_dates,date_parser=date_parser,thousands=thousands,comment=comment,skipfooter=skipfooter,convert_float=convert_float, mangle_dupe_cols=mangle_dupe_cols,**kwds))

def ExcelWriter(path, engine=None, **kwargs):
    return(pd.ExcelWriter(path=path,engine=engine, **kwargs))

def read_json(path_or_buf=None,orient=None,typ="frame",dtype=None,convert_axes=None,convert_dates=True,keep_default_dates=True,numpy=False,precise_float=False,date_unit=None,encoding=None,lines=False,chunksize=None,compression="infer"):
    return(pd.read_json(path_or_buf=path_or_buf,orient=orient,typ=typ,dtype=dtype,convert_axes=convert_axes,convert_dates=convert_dates,keep_default_dates=keep_default_dates,numpy=numpy,precise_float=precise_float,date_unit=date_unit,encoding=encoding,lines=lines,chunksize=chunksize,compression=compression))
    

def json_normalize(data, record_path=None, meta=None, meta_prefix=None, record_prefix=None, errors='raise', sep='.', max_level=None):
    return(pd.json_normalize(data=data, record_path=record_path, meta=meta, meta_prefix=meta_prefix, record_prefix=record_prefix, errors=errors, sep=sep, max_level=max_level))


def read_html(io,match=".+",flavor=None,header=None,index_col=None,skiprows=None, attrs=None, parse_dates=False,thousands=",",encoding=None,decimal=".",converters=None,na_values=None,keep_default_na=True,displayed_only=True):
    return(pd.read_html(io=io,match=match,flavor=flavor,header=header,index_col=index_col,skiprows=skiprows, attrs=attrs,parse_dates=parse_dates,thousands=thousands,encoding=encoding,decimal=decimal,converters=converters,na_values=na_values,keep_default_na=keep_default_na,displayed_only=displayed_only))

def read_hdf(path_or_buf, key=None, mode='r', errors='strict', where=None, start=None, stop=None, columns=None, iterator=False, chunksize=None, **kwargs):
    return(pd.read_hdf(path_or_buf=path_or_buf, key=key, mode=mode, errors=errors, where=where, start=start, stop=stop, columns=columns, iterator=iterator, chunksize=chunksize, **kwargs))


def read_feather(path, columns=None, use_threads=True):
    return(pd.read_feather(path=path, columns=columns, use_threads=use_threads))

def read_parquet(path, engine='auto', columns=None, **kwargs):
    return(pd.read_parquet(path=path, engine=engine, columns=columns, **kwargs))

def read_orc(path, columns=None, **kwargs):
    return(pd.read_orc(path=path, columns=columns, **kwargs))

def read_sas(filepath_or_buffer,format=None,index=None, encoding=None,chunksize=None,iterator=False,):
    return(pd.read_sas(filepath_or_buffer=filepath_or_buffer,format=format,index=index, encoding=encoding,chunksize=chunksize,iterator=iterator,))

def read_spss(path, usecols=None, convert_categoricals=True):
    return(pd.read_spss(path=path, usecols=usecols, convert_categoricals=convert_categoricals))

def read_sql_table(table_name,con,schema=None,index_col=None,coerce_float=True,parse_dates=None,columns=None,chunksize=None,):
    return(pd.read_sql_table(table_name=table_name,con=con,schema=schema,index_col=index_col,coerce_float=coerce_float,parse_dates=parse_dates,columns=columns,chunksize=chunksize,))

def read_sql_query(sql,con,index_col=None,coerce_float=True,params=None,parse_dates=None,chunksize=None,):
    return(pd.read_sql_query(sql=sql,con=con,index_col=index_col,coerce_float=coerce_float,params=params,parse_dates=parse_dates,chunksize=chunksize,))

def read_sql(sql,con,index_col=None,coerce_float=True,params=None,parse_dates=None,columns=None,chunksize=None,):
    return(pd.read_sql(sql=sql,con=con,index_col=index_col,coerce_float=coerce_float,params=params,parse_dates=parse_dates,columns=columns,chunksize=chunksize,))

def read_gbq(query, project_id=None, index_col=None, col_order=None, reauth=False, auth_local_webserver=False, dialect=None, location=None, configuration=None, credentials=None, use_bqstorage_api=None, max_results=None, private_key=None, verbose=None, progress_bar_type=None):
    return(pd.read_gdq(query=query, project_id=project_id, index_col=index_col, col_order=col_order, reauth=reauth, auth_local_webserver=auth_local_webserver, dialect=dialect, location=location, configuration=configuration, credentials=credentials, use_bqstorage_api=use_bqstorage_api, max_results=max_results, private_key=private_key, verbose=verbose, progress_bar_type=progress_bar_type))

def read_stata(filepath_or_buffer, convert_dates=True, convert_categoricals=True, index_col=None, convert_missing=False, preserve_dtypes=True, columns=None, order_categoricals=True, chunksize=None, iterator=False):
    return(pd.read_stata(filepath_or_buffer=filepath_or_buffer, convert_dates=convert_dates, convert_categoricals=convert_categoricals, index_col=index_col, convert_missing=convert_missing, preserve_dtypes=preserve_dtypes, columns=columns, order_categoricals=order_categoricals, chunksize=chunksize, iterator=iterator))



def data_profile(dataframe):
    """
    Parameters:
    dataframe : dataframe

    Returns - Data Dictionary
    """
    # get a list of columns
    columns = list(dataframe) # list of columns
    # get list of unique values in each column
    unique_values = list(dataframe.nunique().values)
    # get list of missing values in each column
    missing_values = list(dataframe.isnull().sum().values)
    # get list of missing value % in each column
    missing_value_percentage = list((dataframe.isnull().sum() * 100)/ len(dataframe))
    # get datatype of each column
    data_types = list(dataframe.dtypes.values)
    
    param = []
    value_list = []
    # for each column, append the param in pandas describe function and the value associated with it - include='all' will give the stats of categorical variables also
    for i in range(len(dataframe.describe(include='all').index)): 
        param.append(dataframe.describe(include='all').index[i])
        value_list.append(list(dataframe.describe(include='all').iloc[i].values))
    
    # outliers - higher/lower
    # calculate outliers for numerical columns 
    higher_outlier = []
    lower_outlier = []
    for i in list(dataframe):
        # check if the datatype of column id int64
        if dataframe[i].dtypes == 'int64':
            higher = dataframe[i].quantile(0.75)+1.5*(dataframe[i].quantile(0.75)-dataframe[i].quantile(0.25))
            lower = dataframe[i].quantile(0.25)-1.5*(dataframe[i].quantile(0.75)-dataframe[i].quantile(0.25))
            higher_outlier.append(higher)
            lower_outlier.append(lower)
        # if not int64 append null values
        else:
            higher_outlier.append(np.nan)
            lower_outlier.append(np.nan)
    # create a dataframe with all the above metrics related to each columna dn return the dataframe
    describe_dict = dict(zip(param,value_list))
    properties_dict = {'columns':columns,'data_types':data_types,'unique_values':unique_values,'missing_values':missing_values,'missing_value_percentage':missing_value_percentage}
    outlier_dict = {"higer": higher_outlier, "lower":lower_outlier}  
    
    describe_dict.update(properties_dict)
    describe_dict.update(outlier_dict)
    
    # put above things in a dataframe
    eda_df = pd.DataFrame(describe_dict)
    return (eda_df)

def treat_missing_data(dataframe, column, **kwargs):
    """
    Parameters:
    dataframe
    column: list of columns/column to treat missing data
    **kwargs: Default - statistic - mean
        give any on of the below parameters - either statistic/value/fill_method
        statistic: min/max/mean/median/quantilevalue
        value: user specified value
        fill_method: bfill/ffill (bfill - backward fill, ffill - front fill)
    """
    # if the parameter in kwargs is not given, take default as statistic-mean
    if len(kwargs) == 0:
        kwargs['statistic'] = 'mean'
    dataframe_duplicate = dataframe.copy()
    # if more than one kwargs is given raise error
    if len(kwargs.keys())>1:
        raise ValueError('please select any one argument value/statistic/fill_method')
    else:
        # if params in kwargs are not valid, raise error
        if list(kwargs.keys())[0] not in ['value','statistic','fill_method']:
            raise ValueError ('please choose a valid argument')
        else:
            new_columns = []
            for i in column:
                # if the variable is categorical, fill missing values with mode
                if dataframe_duplicate[i].dtypes not in ['int64','float64']:
                    kwargs['statistic'] = 'mode'
                else: pass
                # variable for given kwarg - value/statistic/fill_method
                arg = list(kwargs.keys())[0]
                if arg == 'value':
                    # if value - fill missing values with the specified value
                    #dataframe_duplicate[i] = dataframe_duplicate[i].fillna(kwargs[arg])
                    new_columns.append(dataframe_duplicate[i].fillna(kwargs[arg]))

                elif arg == 'statistic':
                    # if statistic - fill with specified statistic
                    statistic_value = dataframe_duplicate[i].mode() if kwargs[arg] == 'mode' else dataframe_duplicate[i].max() if kwargs[arg]=='max' else dataframe_duplicate[i].min() if kwargs[arg]=='min' else dataframe_duplicate[i].mean() if kwargs[arg]=='mean' else dataframe_duplicate[i].median() if kwargs[arg]=='median' else dataframe_duplicate[i].quantile(kwargs[arg]) if (type(kwargs[arg]) == int or type(kwargs[arg]) == float) else 0
                    column_array = dataframe_duplicate[i]
                    new_columns.append(column_array.fillna(statistic_value))
                # if arg is fill_method, front fill or back fill
                elif arg == 'fill_method':
                    #dataframe_duplicate[i] = dataframe_duplicate[i].fillna(method=kwargs[arg])
                    new_columns.append(dataframe_duplicate[i].fillna(method=kwargs[arg]))
                else: pass
    return new_columns

def treat_outliers(dataframe, column, **kwargs):
    """
    Parameters
    dataframe
    column: list of columns/column to treat outliers
    **kwrags - Default - Statistic - mean
        select any one of the below parameters - statistic/value/remove
        remove : True/False - flag to delete the rows with outliers
        statistic - min/max/mean/median/quantilevalue
        value - replaces outliers with user specified value
    """
    dataframe_duplicate = dataframe.copy()
    # params in kwargs
    function_args_list = ['remove','value','statistic','higher','lower']
    user_input_args_list = list(kwargs.keys())
    args_check_flag = all(elem in function_args_list  for elem in user_input_args_list)
    
    if args_check_flag == False:
        raise ValueError('please choose a valid argument')
    datatype_check_flag = all([True if (dataframe_duplicate[i].dtypes=='int64' or dataframe_duplicate[i].dtypes=='float64') else False for i in column])
    if not datatype_check_flag:
        raise Exception('function not applicable for columns of type category')
        
    missing_value_check_flag = all([True if dataframe_duplicate[i].isnull().sum()>1 else False for i in column])
    if missing_value_check_flag:
        raise ValueError('Contains missing values - please use missing_data function to fill missing values')
    
    remove = kwargs.get('remove',False) # takes the given value of remove flag
    print(remove)
    upper = kwargs.get('upper',True)
    print(upper)
    lower = kwargs.get('lower',True)
    print(lower)
    new_column = []
    for i in column:
        
        higher_outlier = dataframe_duplicate[i].quantile(0.75)+1.5*(dataframe_duplicate[i].quantile(0.75)-dataframe_duplicate[i].quantile(0.25))
        print(higher_outlier)
        lower_outlier = dataframe_duplicate[i].quantile(0.25)-1.5*(dataframe_duplicate[i].quantile(0.75)-dataframe_duplicate[i].quantile(0.25))
        print(lower_outlier)
        if remove:
            print('in remove')
            print(kwargs['remove'])
            if kwargs['remove'] not in [True,False]: raise ValueError('remove should be either True/False')
            print(dataframe_duplicate[i])
            if upper:
                dataframe_duplicate = dataframe_duplicate[dataframe_duplicate[i]<=higher_outlier]
            if lower:
                dataframe_duplicate = dataframe_duplicate[dataframe_duplicate[i]>=lower_outlier]
        else:
            if 'value' in kwargs:
                if type(kwargs['value']) != int: raise ValueError('value should be a interger')
                value = kwargs['value']
            elif 'statistic' in kwargs:
                if kwargs['statistic'] in ['min','max','mean','median'] or type(kwargs['statistic'])==int:
                    pass
                else:
                    raise ValueError('statistic should be min/max/mean/median or a quantile value between 0 and 1')
                minimum = dataframe_duplicate[i][(dataframe_duplicate[i]>lower_outlier) & (dataframe_duplicate[i]<higher_outlier)].min()
                maximum = dataframe_duplicate[i][(dataframe_duplicate[i]>lower_outlier) & (dataframe_duplicate[i]<higher_outlier)].max()
                mean = dataframe_duplicate[i][(dataframe_duplicate[i]>lower_outlier) & (dataframe_duplicate[i]<higher_outlier)].mean()
                median = dataframe_duplicate[i][(dataframe_duplicate[i]>lower_outlier) & (dataframe_duplicate[i]<higher_outlier)].median()
                quantile_value = dataframe_duplicate[i][(dataframe_duplicate[i]>lower_outlier) & (dataframe_duplicate[i]<higher_outlier)].quantile(kwargs['statistic']) if (type(kwargs['statistic']) == int or type(kwargs['statistic']) == float) else 0
                    
                value = minimum if kwargs['statistic'] == 'min' else maximum if kwargs['statistic']=='maximum' else mean if kwargs['statistic'] == 'mean' else median if kwargs['statistic'] == 'median' else quantile_value if type(kwargs['statistic']) == int else 0
            else:
                value = dataframe_duplicate[i][(dataframe_duplicate[i]>lower_outlier) & (dataframe_duplicate[i]<higher_outlier)].mean()
                    
            if upper:
                dataframe_duplicate[i] = np.where(dataframe_duplicate[i]>=higher_outlier,value,dataframe_duplicate[i])
            if lower:
                dataframe_duplicate[i] = np.where(dataframe_duplicate[i]<=lower_outlier,value,dataframe_duplicate[i])
    
    return dataframe_duplicate

def feature_creation(dataframe,feature_method, binning_column = None, bins = 10,binning_right=True,binning_labels=None,binning_retbins = False, binning_precision = 3,binning_include_lowest = False,binning_duplicates='raise',binning_ordered=True,dummies_prefix=None, dummies_prefix_sep='_', dummies_dummy_na=False, dummies_column=None, dummies_sparse=False, dummies_drop_first = False,groupby_column = None, groupby_transform_column = None,groupby_transform_metric='mean'):
    """
    Parameters:
    dataframe
    feature_method: binning/one-hot-encoding/groupby
    params if feature_method is binning: The criteria to bin by.
        bins - default int 10 - 
            int : Defines the number of equal-width bins in the range of x. The range of x is extended by .1% on each side to include the minimum and maximum values of x
            sequence of scalars : Defines the bin edges allowing for non-uniform width. No extension of the range of x is done.
            IntervalIndex : Defines the exact bins to be used.
        binning_right: bool, default True
            Indicates whether bins includes the rightmost edge or not. If right == True (the default), then the bins [1, 2, 3, 4] indicate (1,2], (2,3], (3,4]. This argument is ignored when bins is an IntervalIndex.
        binning_labels: array or bool, optional
            Specifies the labels for the returned bins. Must be the same length as the resulting bins. If False, returns only integer indicators of the bins. This affects the type of the output container (see below). This argument is ignored when bins is an IntervalIndex.
        binning_retbins: bool, default False
            Whether to return the bins or not. Useful when bins is provided as a scalar.
        binning_precision:  int, default 3
            The precision at which to store and display the bins labels.
        binning_include_lowest: bool, default False
            Whether the first interval should be left-inclusive or not.
        binning_duplicates: {default ‘raise’, ‘drop’}, optional
            If bin edges are not unique, raise ValueError or drop non-uniques.
    params if feature_method is one-hot-encoding: Convert categorical variable into dummy/indicator variables.
        dummies_prefix - str, list of str, or dict of str, default None
            String to append DataFrame column names. Pass a list with length equal to the number of columns when calling get_dummies on a DataFrame. Alternatively, prefix can be a dictionary mapping column names to prefixes.
        dummies_prefix_sep: str, default ‘_’
            If appending prefix, separator/delimiter to use. Or pass a list or dictionary as with prefix.
        dummies_dummy_na: bool, default False
            Add a column to indicate NaNs, if False NaNs are ignored.
        dummies_column: list-like, default None
            Column names in the DataFrame to be encoded. If columns is None then all the columns with object or category dtype will be converted.
        dummies_sparse: bool, default False
            Whether the dummy-encoded columns should be backed by a SparseArray (True) or a regular NumPy array (False).
        dummies_drop_first: bool, default False
            Whether to get k-1 dummies out of k categorical levels by removing the first level.
    params if feature_method is groupby: groups by the specified columns, gets the metrics of the transform column in each group. Returns a new column
        groupby_column - list of column/columns
        groupby_transform_column - column to apply the metric on 
        groupby_transform_metric - min/max/mean/median
    """
    if feature_method == 'binning':
        # if binning is selected and binning column is not specified - raise error
        if binning_column == None:
            raise ValueError('please specify a column for binning')
        else:
            # call pandas pd.cut function with user specified params
            return(pd.cut(dataframe[binning_column], bins=bins, right = binning_right, labels=binning_labels, retbins = binning_retbins, precision=binning_precision, include_lowest=binning_include_lowest, duplicates=binning_duplicates))
    elif feature_method == 'one-hot-encoding':
        # call get_dummies
        return(pd.get_dummies(dataframe, prefix = dummies_prefix, prefix_sep = dummies_prefix_sep, dummy_na = dummies_dummy_na, columns = dummies_column, sparse = dummies_sparse, drop_first=dummies_drop_first))
    elif feature_method == 'groupby':
        if groupby_column == None or groupby_transform_column == None:
            raise ValueError('please specify a valid groupby_column/groupby_transform_column')
        group_column = groupby_column if type(groupby_column) == list else [groupby_column]
        return(dataframe.groupby(group_column)[groupby_transform_column].transform(groupby_transform_metric))
    else: pass

def data_transformation(dataframe,transform_method, pivot_index=None, pivot_columns=None,pivot_values=None, melt_id_vars=None, melt_value_vars=None, crosstab_columns = None, crosstab_rows = None):
    """
    Parameters
    dataframe
    transform_method: pivot/melt/crosstab
    params if transform_method is pivot
        Return reshaped DataFrame organized by given index / column values.

        Reshape data (produce a “pivot” table) based on column values. Uses unique values from specified index / columns to form axes of the resulting DataFrame. This function does not support data aggregation, multiple values will result in a MultiIndex in the columns. See the User Guide for more on reshaping.

        pivot_index: str or object or a list of str, optional
            Column to use to make new frame’s index. If None, uses existing index.

        pivot_columns: str or object or a list of str
            Column to use to make new frame’s columns.

        pivot_values: str, object or a list of the previous, optional
            Column(s) to use for populating new frame’s values. If not specified, all remaining columns will be used and the result will have hierarchically indexed columns.
    params if transform_method is melt:
        Unpivot a DataFrame from wide to long format, optionally leaving identifiers set.

        This function is useful to massage a DataFrame into a format where one or more columns are identifier variables (id_vars), while all other columns, considered measured variables (value_vars), are “unpivoted” to the row axis, leaving just two non-identifier columns, ‘variable’ and ‘value’.

        melt_id_vars: tuple, list, or ndarray, optional
            Column(s) to use as identifier variables.
        melt_value_vars: tuple, list, or ndarray, optional
            Column(s) to unpivot. If not specified, uses all columns that are not set as id_vars.

    params if transform_method is crosstab
        crosstab_rows: array-like, Series, or list of arrays/Series
            Values to group by in the rows.
        crosstab_columns: array-like, Series, or list of arrays/Series.
            Values to group by in the columns.
        """
    if transform_method not in ['pivot','melt','crosstab']:
        raise ValueError('transform_method should be either pivot/melt/crosstab')
    if transform_method == 'pivot':
        if pivot_index == None or pivot_columns == None:
            raise ValueError('pivot_index and pivot_columns are needed for transform_method pivot')
        else:
            return(dataframe.pivot(index=pivot_index,columns=pivot_columns,values=pivot_values))
    elif transform_method == 'melt':
        if melt_id_vars==None or melt_value_vars==None:
            raise ValueError('melt_id_vars and melt_value_cars are needed for transform_method melt')
        else:
            return(dataframe.melt(id_vars=melt_id_vars,value_vars=melt_value_vars))
    elif transform_method == 'crosstab':
        if crosstab_columns==None or crosstab_rows==None:
            raise ValueError('crosstab_columns and crosstab_rows are needed for transform_method crosstab')
        else:
            crosstab_rows = crosstab_rows if type(crosstab_rows)!=list else crosstab_rows[0]
            crosstab_columns = crosstab_columns if type(crosstab_columns)!=list else crosstab_columns[0]
            return(pd.crosstab(dataframe[crosstab_rows],dataframe[crosstab_columns]))
    else: pass



# -*- coding: utf-8 -*-
"""
Created on Thu Nov  5 08:11:25 2020

@author: Harika
"""
#Importi
import pandas as pd
import plotly
import plotly.graph_objs as go
import plotly.figure_factory as ff
from plotly.offline import download_plotlyjs, init_notebook_mode, plot, iplot
import plotly.express as px     
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns

    
    
# Bar graph
def bar_graph(library, dataset_path, datasource_type, values_list_column, target_column, main_title, title_x_axis, title_y_axis):
    """
    Parameters:
    library : plotly/matplotlib/seaborn
    dataset_path : Path of dataset
    datasource_type : csv/excel/dataframe
    values_list_column : list of independent coumns mention
    target_column : dependent column mention
    main_title : Main title of graph
    title_x_axis : X_axis title of graph
    title_y_axis : Y_axis title of graph
    """
    if library == "plotly":
     
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
        
        new_data = pd.pivot_table(dataset, values=values_list_column, columns= target_column)
        traces = [go.Bar(
        x = new_data.columns,
        y = new_data.loc[rowname],
        name = rowname
        )for rowname in new_data.index]
        layout = go.Layout(title = main_title)
        fig = plotly.graph_objs.Figure(data = traces,layout = layout)
        fig.update_layout(
            xaxis_title=title_x_axis,
            yaxis_title=title_y_axis)
        plotly.offline.iplot(fig)
        
    elif library =="matplotlib":
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
            
        # Figure Size 
        fig = plt.figure(figsize =(10, 7)) 
        value = dataset[values_list_column]
        target = dataset[target_column]
        # Horizontal Bar Plot 
        plt.bar(value, target)
        plt.title(main_title)
        plt.xlabel(title_x_axis) 
        plt.ylabel(title_y_axis) 
    
        plt.show()
        
    elif library == "seaborn":
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
      
        sns.barplot(x = values_list_column, y = target_column, data = dataset) 
        plt.title(main_title)
        plt.xlabel(title_x_axis) 
        plt.ylabel(title_y_axis) 
        plt.show() 
    
    else:
        print("incorrect library used")



#line graph
def line_graph(library, dataset_path, datasource_type, values_list_column, target_column, main_title, title_x_axis, title_y_axis):
    """
    Parameters:
    library : plotly/matplotlib/seaborn
    dataset_path : Path of dataset
    datasource_type : csv/excel/dataframe
    values_list_column : list of independent coumns mention
    target_column : dependent column mention
    main_title : Main title of graph
    title_x_axis : X_axis title of graph
    title_y_axis : Y_axis title of graph
    """
    if library == "plotly":
        
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
    
        new_data = pd.pivot_table(dataset, values=values_list_column, columns= target_column)
        traces = [go.Line(
        x = new_data.columns,
        y = new_data.loc[rowname],
        mode = 'lines',
        name = rowname
        )for rowname in new_data.index]
        layout = go.Layout(title = main_title)
        fig = plotly.graph_objs.Figure(data = traces,layout = layout)
        fig.update_layout(
            xaxis_title=title_x_axis,
            yaxis_title=title_y_axis)
        plotly.offline.iplot(fig)
        
        
    elif library == "matplotlib":
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
            
        
        plt.plot(dataset[values_list_column], dataset[target_column])
        plt.title(main_title)
        plt.xlabel(title_x_axis)
        plt.ylabel(title_y_axis)
        plt.show()
        
    elif library == "seaborn":
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
  
        plt.figure(figsize=(15,15))
  
        plt.title(main_title)
        
        sns.lineplot(dataset[values_list_column],dataset[target_column])
  
        plt.xlabel(title_x_axis)
        plt.ylabel(title_y_axis)
        plt.show()
        
    else:
        print("incorrect library used")



# scatter plot
def scatter_plot(library, dataset_path, datasource_type, values_list_column, target_column, main_title, title_x_axis, title_y_axis):
    
    """
    Parameters:
    library : plotly/matplotlib/seaborn
    dataset_path : Path of dataset
    datasource_type : csv/excel/dataframe
    values_list_column : list of independent coumns mention
    target_column : dependent column mention
    main_title : Main title of graph
    title_x_axis : X_axis title of graph
    title_y_axis : Y_axis title of graph
    """
    if library == "plotly": 
        
        
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
    
        new_data = pd.pivot_table(dataset, values=values_list_column, columns= target_column)
        traces = [go.Scatter(
        x = new_data.columns,
        y = new_data.loc[rowname],
        mode = 'markers',
        name = rowname
        )for rowname in new_data.index]
        layout = go.Layout(title = main_title)
        fig = plotly.graph_objs.Figure(data = traces,layout = layout)
        fig.update_layout(
            xaxis_title=title_x_axis,
            yaxis_title=title_y_axis)
        plotly.offline.iplot(fig)
   
        
   
    elif library == "matplotlib":
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
    
  
        x = dataset[values_list_column]
        y = dataset[target_column]
       
        
        plt.scatter(x, y, cmap='viridis', alpha=0.3)
        plt.xlabel(title_x_axis)
        plt.ylabel(title_y_axis)
        plt.show()
        
        
        
    elif library == "seaborn":
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
        
        # Set the width and height of the figure
        plt.figure(figsize=(14,6))
        
        # Add title
        plt.title(main_title)
        sns.scatterplot(data=dataset, x=values_list_column, y=target_column)
    
        
        # Add label for horizontal axis
        plt.xlabel(title_x_axis)
        plt.ylabel(title_y_axis)
        plt.show()
        
    else :
        print("incorrect library used")
      
  
# Pie graph
def pie_graph(library, dataset_path, datasource_type, values_list_column, main_title):
    """
    Parameters:
    library : plotly/matplotlib/seaborn
    dataset_path : Path of dataset
    datasource_type : csv/excel/dataframe
    values_list_column : list of independent coumns mention
    main_title : Main title of graph
    """
    
    if library=="plotly":
        
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
        trace = plotly.graph_objs.Pie(values=dataset[values_list_column].value_counts().values.tolist(),
                                      labels=dataset[values_list_column].value_counts().keys().tolist(),
                                      hoverinfo="label+percent+name",
                                      domain=dict(x=[0, .48]),
                                      marker=dict(line=dict(width=2,
                                                            color="rgb(243,243,243)")
                                                  ),
                                      hole=.6
                                      )
    
        layout = plotly.graph_objs.Layout(dict(title=values_list_column,
                                                plot_bgcolor="rgb(243,243,243)",
                                                paper_bgcolor="rgb(243,243,243)",
                                                annotations=[dict(text=main_title,
                                                                  font=dict(size=13),
                                                                  showarrow=False,
                                                                  x=.15, y=.5),
    
                                                            ]
                                                )
                                          )
        data = [trace]
        fig = plotly.graph_objs.Figure(data=data, layout=layout)
        plotly.offline.iplot(fig)
        
    elif library == "matplotlib":
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
        values = dataset[values_list_column].value_counts()
       
        per=[]
        for i in values:
            perc = i/values.sum()
            per.append(format(perc,'.2f'))
            
        plt.figure(figsize=(10,6))    
        plt.title(main_title,fontsize=20)
        plt.pie(per,autopct='%1.1f%%')
        plt.legend(values.index,loc='best', fontsize=15)
    
    
    elif library == "seaborn":
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
        
        pie, ax = plt.subplots(figsize=[10,6])
        values = dataset[values_list_column].value_counts().values.tolist()
        labels = dataset[values_list_column].value_counts().keys()
  
        plt.pie(x=values, autopct="%.1f%%", labels=labels, pctdistance=0.5)
        plt.title(main_title, fontsize=14);
        
                    
                    
    else:
        print("incorrect library used")
    

#Box Plot
def box_plot(library, dataset_path, datasource_type, values_list_column, target_column, main_title, title_x_axis, title_y_axis):
    """
    Parameters:
    library : plotly/matplotlib/seaborn
    dataset_path : Path of dataset
    datasource_type : csv/excel/dataframe
    values_list_column : list of independent coumns mention
    target_column : dependent column mention
    main_title : Main title of graph
    title_x_axis : X_axis title of graph
    title_y_axis : Y_axis title of graph
    """
    if library == "plotly": 
        
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
    
        fig = px.box(dataset, x=values_list_column, y=target_column)
        fig.show()
                      
          
    elif library == "matplotlib":
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
        
        
        dataset.boxplot(by = values_list_column, column = target_column, grid = False) 
        
        plt.title(main_title)
       
        plt.xlabel(title_x_axis)
        plt.ylabel(title_y_axis)
        plt.show()
          
          
          
    elif library == "seaborn":
    
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
    
 
        plt.figure(figsize=(14,6))
    
        # Add title
        plt.title(main_title)
        ax= sns.boxplot(x=values_list_column,y=target_column,data=dataset)
       
        # Add label for horizontal axis
        plt.xlabel(title_x_axis)
        plt.ylabel(title_y_axis)
        plt.show()
          
          
    else:
        print("incorrect library used")
      
      
  
  
#Violin Plot
def violin_plot(library, dataset_path, datasource_type, values_list_column, target_column, main_title, title_x_axis, title_y_axis):
    """
    Parameters:
    library : plotly/matplotlib/seaborn
    dataset_path : Path of dataset
    datasource_type : csv/excel/dataframe
    values_list_column : list of independent coumns mention
    target_column : dependent column mention
    main_title : Main title of graph
    title_x_axis : X_axis title of graph
    title_y_axis : Y_axis title of graph
    """
  
    if library == "matplotlib":
        
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
    
        data_to_plot = [dataset[values_list_column], dataset[target_column]]
        
        # Create a figure instance
        fig = plt.figure()
        
        # Create an axes instance
        ax = fig.add_axes([0,0,1,1])
        
        # Create the boxplot
        bp = ax.violinplot(data_to_plot)
        plt.title(main_title)
         
        plt.xlabel(title_x_axis)
        plt.ylabel(title_y_axis)
        plt.show()
        plt.show() 
          
         
        
    elif library == "seaborn":
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
            
        sns.violinplot(x=values_list_column, y=target_column, data=dataset, palette="Pastel1")
        plt.title(main_title)
       
        plt.xlabel(title_x_axis)
        plt.ylabel(title_y_axis)
        plt.show()
      
        
    elif library == "plotly":
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
            
        
        fig1 = px.violin(dataset, y=values_list_column, x=target_column)
        
        fig1.show()
        
    
    else:
        print("incorrect library used")
                  
          
                  
  
# Function for correlation  matrix
def correlation_matrix(library, dataset_path, datasource_type, main_title, scale_title):
    """
    Parameters:
    library : plotly/matplotlib/seaborn
    dataset_path : Path of dataset
    datasource_type : csv/excel/dataframe
    main_title : Main title of graph
    scale_title: Title of the scale mention
    """
    
    if library=="plotly":
        
      
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
  
        correlation = dataset.corr()
        matrix_cols = correlation.columns.tolist()
        corr_array = np.array(correlation)
  
        trace = plotly.graph_objs.Heatmap(z=corr_array,
                                          x=matrix_cols,
                                          y=matrix_cols,
                                          colorscale="Viridis",
                                          colorbar=dict(title=scale_title,
                                                        titleside="right"
                                                        ),
                                          )
  
        layout = plotly.graph_objs.Layout(dict(title=main_title,
                                                autosize=False,
                                                height=720,
                                                width=800,
                                                margin=dict(r=0, l=210,
                                                            t=25, b=210,
                                                            ),
                                                yaxis=dict(tickfont=dict(size=9)),
                                                xaxis=dict(tickfont=dict(size=9))
                                                )
                                          )
  
        data = [trace]
        fig = plotly.graph_objs.Figure(data=data, layout=layout)
        plotly.offline.iplot(fig)
        
  
    elif library=="matplotlib":
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
  
        corr = dataset.corr()
  
        from matplotlib import rc
        rc('font',**{'family':'sans-serif','sans-serif':['Arial']})
        plt.suptitle(main_title, fontsize=16)
        
        plt.pcolor(corr, cmap='RdBu_r')
        cb = plt.colorbar()
        cb.set_label(scale_title, fontsize=14)
        plt.show()
        
        
    elif library=="seaborn":
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
            
        corr = dataset.corr()   
          
        plt.figure(figsize=(16, 6))
  
        heatmap = sns.heatmap(corr, vmin=-1, vmax=1, annot=True)
  
        heatmap.set_title(main_title, fontdict={'fontsize':12}, pad=12);
        plt.show()
        
    else:
        print("incorrect library used")




#Distribution Plot
def dist_plot(library, dataset_path, datasource_type, values_list_column, main_title):
    """
    Parameters:
    library : plotly/matplotlib/seaborn
    dataset_path : Path of dataset
    datasource_type : csv/excel/dataframe
    values_list_column : list of independent coumns mention
    main_title : Main title of graph
    """
    
    if library == "seaborn":
        
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
        sns.set(rc={'figure.figsize':(11.7,8.27)})
        sns.distplot(dataset[values_list_column], bins=30)
        plt.title(main_title)
        plt.show()
      
    elif library == "matplotlib":
        
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
        
        dataset[values_list_column].hist(bins=15, figsize=(11,11))
        plt.title(main_title)
        plt.show()
      
    elif library == "plotly":
        
        if datasource_type == 'csv':
            dataset = pd.read_csv(dataset_path)
        elif datasource_type == 'excel':
            dataset = pd.read_excel(dataset_path)
        elif datasource_type == 'dataframe':
            dataset = dataset_path
        
        
        hist_data = [dataset[values_list_column]]
        group_labels = [main_title] # name of the dataset
        
        fig = ff.create_distplot(hist_data, group_labels)
        fig.show()

    else:
        print("incorrect library used")
          
      
      

   
#Single lib plots      
def histogram(dataset_path, datasource_type):
    """
    Parameters:
    dataset_path : Path of dataset
    datasource_type : csv/excel/dataframe
    """
    
    if datasource_type == 'csv':
        dataset = pd.read_csv(dataset_path)
    elif datasource_type == 'excel':
        dataset = pd.read_excel(dataset_path)
    elif datasource_type == 'dataframe':
        dataset = dataset_path
    dataset.hist(bins=15, figsize=(5,5))
    plt.show()
  
  
  
#To Check the missingvalues using heatmap
def missingvalues(dataset_path, datasource_type):
    """
    Parameters:
    dataset_path : Path of dataset
    datasource_type : csv/excel/dataframe
    """
  
    if datasource_type == 'csv':
        dataset = pd.read_csv(dataset_path)
    elif datasource_type == 'excel':
        dataset = pd.read_excel(dataset_path)
    elif datasource_type == 'dataframe':
        dataset = dataset_path
        
    heatmap = sns.heatmap(dataset.isnull(),cbar=False, yticklabels=False, cmap = 'viridis')
    heatmap.set_title('Missing Values Check', fontdict={'fontsize':12}, pad=12);
    plt.show()


# to check count 
def count_plot(dataset_path, datasource_type, values_list_column, main_title):
    """
    Parameters:
    dataset_path : Path of dataset
    datasource_type : csv/excel/dataframe
    values_list_column : list of independent coumns mention
    main_title : Main title of graph
    """
 
    if datasource_type == 'csv':
        dataset = pd.read_csv(dataset_path)
    elif datasource_type == 'excel':
        dataset = pd.read_excel(dataset_path)
    elif datasource_type == 'dataframe':
        dataset = dataset_path
    sns.set(rc={'figure.figsize':(12,9)})
    sns.countplot(x = values_list_column, data = dataset, palette = "Set2") 
    plt.title(main_title)
    plt.show()
    # Show the plot 
    plt.show() 
   
                

   


# %%
import os
import glob
import numpy as np
from skimage.io import imread, imsave
import matplotlib.pyplot as plt
from skimage import transform, data , color, img_as_float, segmentation, filters
from skimage.transform import rotate, AffineTransform, rescale, resize, downscale_local_mean
from skimage.util import random_noise
from skimage.filters import gaussian
from scipy import ndimage

from skimage.color import (separate_stains, combine_stains,hdx_from_rgb,rgb2xyz,rgb2gray, rgb_from_hdx,gray2rgb, convert_colorspace)

from skimage.restoration import (denoise_tv_chambolle, denoise_bilateral,denoise_wavelet, estimate_sigma)


from skimage.morphology import erosion, dilation, opening, closing, white_tophat
#%%
def extract_images(folder_path):
  data_path = os.path.join(folder_path,'*g') 
  files = glob.glob(data_path) 
  imgfiles=[]
  for files in files:
    if (files.endswith(".jpeg")==True)|(files.endswith(".jpg")==True):
      imgfiles.append(files)
  return imgfiles
# %% Transformation
def image_transformation_scikit(image_path,is_image,method,save_image = False):
    # if user selects a invalid method we raise an error
    if method not in ['rotate_clockwise_90','rotate_anticlockwise_90','rotate_180']:
        raise ValueError('Select valid parameter for method- rotate_clockwise_90/rotate_anticlockwise_90/rotate_180')
    if save_image not in [True,False]:
        save_image=False
    # if user inputs single image, check for only image formats and pass for further processing
    if is_image == True:
        image = imread(image_path)
        if (image_path.endswith('.jpeg') or image_path.endswith('.png') or image_path.endswith('.jpg')): pass
        else: raise ValueError('please provide a valid image path for is_image=True')
    #image passes through the mentioned method for transformation 
        if method == 'rotate_clockwise_90':
            rotated_list = rotate(image,90,resize=True)
            if save_image==True:
                imsave('image_rotate_clockwise_90.jpg', rotated_list)
            
        elif method == 'rotate_anticlockwise_90':
            rotated_list = rotate(image,270,resize=True)
            if save_image==True:
                imsave('image_rotate_anti_clockwise_90.jpg', rotated_list)
       
        elif method == 'rotate_180':
            rotated_list = rotate(image,180,resize=True)
          #save_image= True will save the image file to the woring directory to view   
            if save_image==True:
                imsave('image_rotate_anti_clockwise_90.jpg', rotated_list)
    #if is_image== False, loop continuous to process and check for only image format files for further processing
    elif is_image == False:
        if (image_path.endswith('.jpeg') or image_path.endswith('.png') or image_path.endswith('.jpg')): pass
        else: pass
        #creating new list to append all the processed content into rotated_list=[]
        rotated_list= []
        #image_count= 1 to intialize to 1 which will get incrimented everytime when a new data enters the list.
        image_count= 1
        image_path_list = extract_images(image_path)
        for i in image_path_list:
            image_read = imread(i)
        ##image passes through the mentioned method for transformation   
            if method == 'rotate_clockwise_90':
                #rotated = rotate(image_read,method=90)
                rotated = rotate(image_read, 90, resize=True)
                if save_image==True:
                    imsave('image_{}_resize-rotate_clockwise_90.jpg'.format(image_count), rotated)
                    print('saved')
                rotated_list.append(rotated)
            elif method == 'rotate_anticlockwise_90':
                rotatedacc = rotate(image_read,270,resize=True)
                if save_image==True:
                    imsave('image_{}_resize-rotate_anticlockwise_90.jpg'.format(image_count), rotatedacc)
                    print('saved')
                rotated_list.append(rotatedacc)
            elif method == 'rotate_180':
                rotatedoo = rotate(image_path,180,resize=True)
                if save_image==True:
                    imsave('image_{}_resize-rotate_180.jpg'.format(image_count), rotatedoo)
                    print('saved')
                rotated_list.append(rotatedoo)
            else:
                print('wrong')
         #image_count gets incrimented everytime a new file gets processed.       
            image_count+=1
   #retrning the appended list of processed image data
    return rotated_list
#%%
#image_transformation_scikit('C:\\Users\\kavya\\Desktop\\Kavya\\preprocessing_wrappers\\scikit_opencv\\image_data\\elep.jpg',True,'rotate_clockwise_90',True)

# %%
#colorspace
#colorspace
def image_colorspace_scikit(image_path,is_image,method,save_image=False):
  # if user selects a invalid method we raise an error and default save image to False is the used provides codition other than true / false.
    if method not in ['red','green','yellow','grey','blue']:
        raise ValueError('choose color in red/green/yellow/grey/blue')
    if save_image not in [True,False]:
        save_image=False
    # if user inputs single image, check for only image formats and pass for further processing
    if is_image == True:
        image = imread(image_path)
        if (image_path.endswith('.jpeg') or image_path.endswith('.png') or image_path.endswith('.jpg')): pass
        else: raise ValueError('please provide a valid image path for is_image=True')
      #image passes through the mentioned method for color change and for every save_image= True image saves.
        if method == 'red':
            red_multiplier = [1, 0, 0]
            color_list = red_multiplier * image
            if save_image==True:
                imsave('image_red.jpg', color_list)
        elif method =='green':
            green_multiplier= [0,1,0]
            color_list = green_multiplier* image
            if save_image==True:
                imsave('image_green.jpg', color_list)
        elif method == 'yellow':
            yellow_multiplier= [1,1,0]
            color_list= yellow_multiplier*image
            if save_image==True:
                imsave('image_yellow.jpg', color_list)
        elif method == 'grey':
            color_list = rgb2gray(image)
            if save_image==True:
                imsave('image_grey.jpg', color_list)
        elif method == 'blue':
            blue_multiplier = [0,0,1]
            color_list= blue_multiplier*image
            if save_image==True:
                imsave('image_blue.jpg', color_list)  
  #if is_image== False, loop continuous to process and check for only image format files for further processing          
    elif is_image == False:
        if (image_path.endswith('.jpeg') or image_path.endswith('.png') or image_path.endswith('.jpg')): pass
        else: pass
     #creating new list to append all the processed content into color_list=[]   
        color_list= []
     #image_count= 1 to intialize to 1 which will get incrimented everytime when a new data enters the list.
        image_count=1
     #extract_images function gets activated and images are read and stored into image_path_list
        image_path_list = extract_images(image_path)
        for i in image_path_list:
            image_read = imread(i)
     #image passes through the mentioned method for color change
            if method == 'red':
                red_multiplier = [1, 0, 0]
                rdata = red_multiplier * image_read
                if save_image==True:
                    imsave('image_{}_color_red.jpg'.format(image_count), rdata)
                    print('saved')
                color_list.append(rdata)
            elif method =='green':
                green_multiplier= [0,1,0]
                gdata = green_multiplier* image_read
                if save_image==True:
                    imsave('image_{}_color_green.jpg'.format(image_count), gdata)
                    print('saved')
                color_list.append(gdata)
            elif method== 'yellow':
                yellow_multiplier= [1,1,0]
                ydata= yellow_multiplier*image_read
                if save_image==True:
                    imsave('image_{}_color_yellow.jpg'.format(image_count), ydata)
                    print('saved')
                color_list.append(ydata)
            elif method == 'grey':
                color_list = rgb2gray(image_read)
                if save_image==True:
                    imsave('image_{}_color_grey.jpg'.format(image_count), color_list)
            elif method == 'blue':
                blue_multiplier = [0,0,1]
                color_list= blue_multiplier*image_read
                if save_image==True:
                    imsave('image_{}_color_blue.jpg'.format(image_count), color_list)   
            else:
                print('wrong input operation')
            print(image_count)
        #image_count gets incrimented everytime a new file gets processed.
            image_count+=1
   #retrning the appended list of processed image data
    return color_list
#%%
#image_colorspace_scikit('C:\\Users\\kavya\\Desktop\\Kavya\\preprocessing_wrappers\\scikit_opencv\\image_data',False,'red',True)
#%%
# resize
def image_resize_scikit(image_path,is_image,method,save_image=False):
    # if user selects a invalid method we raise an error
    if method not in['stretch_vertical', 'stretch_horizontal','resize_small']: raise ValueError('Please input correct method. Valid inputs: ''stretch_vertical', 'stretch_horizontal','resize_small')  
    # if user inputs single image, check for only image formats and pass for further processing
    if is_image == True:
        image = imread(image_path)
        if (image_path.endswith('.jpeg') or image_path.endswith('.png') or image_path.endswith('.jpg')): pass
        else: raise ValueError('please provide a valid image path for is_image=True')
    #image passes through the mentioned method for resizing
    #save_image= True will save the image file to the woring directory to view 
        if method == 'stretch_vertical':
            resized_image = resize(image, (image.shape[0] // 2, image.shape[1] // 1), anti_aliasing=True)
            if save_image==True:
                imsave('image_stretch_vertical.jpg', resized_image)
        elif method== 'stretch_horizontal':
            resized_image = resize(image, (image.shape[0] // 2, image.shape[1] // 4), anti_aliasing=True)
            if save_image==True:
                imsave('image_stretch_horizontal.jpg', resized_image)
        elif method == 'resize_small':
            resized_image = resize(image, (image.shape[0] // 1, image.shape[1] // 1), anti_aliasing=True)
            if save_image==True:
                imsave('image_resize_small.jpg', resized_image)
        else:
            print('wrong')
   #if is_image== False, loop continuous to process and check for only image format files for further processing 
    elif is_image == False:
        if (image_path.endswith('.jpeg') or image_path.endswith('.png') or image_path.endswith('.jpg')): pass
        else: pass
        #creating new list to append all the processed content into rotated_list=[]
        resized_image = []
        #image_count= 1 to intialize to 1 which will get incrimented everytime when a new data enters the list.
        image_count=1
        #extract_images function gets activated and images are read and stored into image_path_list
        image_path_list = extract_images(image_path)
        for i in image_path_list:
            image_read = imread(i)
            ##image passes through the mentioned method for resizing
            if method == 'stretch_vertical':
                stretch_vertical = resize(image_read, (image_read.shape[0] // 2, image_read.shape[1] // 1), anti_aliasing=True)
                if save_image==True:
                    imsave('image_{}_stretch_vertical.jpg'.format(image_count),stretch_vertical)
                    print('saved')
                resized_image.append(stretch_vertical)
            elif method== 'stretch_horizontal':
                stretch_hori = resize(image_read, (image_read.shape[0] // 2, image_read.shape[1] // 4), anti_aliasing=True)
                if save_image==True:
                    imsave('image_{}_stretch_horizontal.jpg'.format(image_count),stretch_hori)
                    print('saved')
                resized_image.append(stretch_hori)
            elif method == 'resize_small':
                resize_small = resize(image_read, (image_read.shape[0] // 1, image_read.shape[1] // 1), anti_aliasing=True)
                if save_image==True:
                    imsave('image_{}_resize_small.jpg'.format(image_count),resize_small)
                    print('saved')
                resized_image.append(resize_small)
            #image_count gets incrimented everytime a new file gets processed.
            image_count+=1
    print(resized_image)
    #retrning the appended list of processed image data        
    return resized_image
#%%
#image_resize_scikit('C:\\Users\\kavya\\Desktop\\Kavya\\preprocessing_wrappers\\scikit_opencv\\image_data\\elep.jpg',True,'stretch_vertical',False)



# %%
#smoothing
#smoothing
def image_smoothing_scikit(image_path,is_image,method,save_image=False):
  # if user selects a invalid method we raise an error
    if method not in['denoise_tv_chambolle','denoise_bilateral','denoise_wavelet']: raise ValueError('Please input correct Operation. Valid inputs: ''denoise_tv_chambolle','denoise_bilateral','denoise_wavelet')  
    # if user inputs single image, check for only image formats and pass for further processing
    if is_image == True:
        image = imread(image_path)
        if (image_path.endswith('.jpeg') or image_path.endswith('.png') or image_path.endswith('.jpg')): pass
        else: raise ValueError('please provide a valid image path for is_image=True')
        #standard scikit, skimage.restoration module values and fnctions used
        sigma = 0.155
        noisy = random_noise(image, var=sigma**2)
        sigma_est = estimate_sigma(noisy, multichannel=True, average_sigmas=True)
        #image passes through the mentioned method for SMOOTHING
        #save_image= True will save the image file to the woring directory to view 
        if method == 'denoise_tv_chambolle':
            smooth_list = denoise_tv_chambolle(noisy, weight=0.1, multichannel=True)
            if save_image==True:
                imsave('image_denoise_tv_chambolle.jpg', smooth_list)
      
        elif method == 'denoise_bilateral':
            print('entered denoise_bilateral')
            smooth_list = denoise_bilateral(noisy, sigma_color=0.1, sigma_spatial=15, multichannel=True)
            if save_image==True:
                imsave('image_denoise_bilateral.jpg', smooth_list)
      
        elif method == 'denoise_wavelet':
            smooth_list= denoise_wavelet(noisy, multichannel=True, convert2ycbcr=True, rescale_sigma=True)
            if save_image==True:
                imsave('image_denoise_wavelet.jpg', smooth_list)
   #if is_image== False, loop continuous to process and check for only image format files for further processing   
    elif is_image == False:
        if (image_path.endswith('.jpeg') or image_path.endswith('.png') or image_path.endswith('.jpg')): pass
        else: pass
        #creating new list to append all the processed content into rotated_list=[]
        smooth_list = []
        #extract_images function gets activated and images are read and stored into image_path_list
        image_path_list = extract_images(image_path)
        #image_count= 1 to intialize to 1 which will get incrimented everytime when a new data enters the list.
        image_count = 1
        for i in image_path_list:
            image_read = imread(i)
            sigma = 0.155
            noisy = random_noise(image_read, var=sigma**2)
            sigma_est = estimate_sigma(noisy, multichannel=True, average_sigmas=True)
            #image passes through the mentioned method for smoothing 
            if method == 'denoise_tv_chambolle':
                dn1= denoise_tv_chambolle(noisy, weight=0.1, multichannel=True)
                if save_image==True:
                    imsave('image_{}_denoise_tv_chambolle.jpg'.format(image_count), dn1)
                    print('saved')
                smooth_list.append(dn1)
            elif method == 'denoise_bilateral':
                print('entered denoise_bilateral')
                dnbill2 = denoise_bilateral(noisy, sigma_color=0.1, sigma_spatial=15, multichannel=True)
                if save_image==True:
                    imsave('image_{}_denoise_bilateral.jpg'.format(image_count),dnbill2)
                    print('saved')
                smooth_list.append(dnbill2)
            elif method == 'denoise_wavelet':
                dnwave2= denoise_wavelet(noisy, multichannel=True, convert2ycbcr=True, rescale_sigma=True)
                if save_image==True:
                    imsave('image_{}_denoise_wavelet.jpg'.format(image_count),dnwave2)
                    print('saved')
                smooth_list.append(dnwave2)
            #image_count gets incrimented everytime a new file gets processed.
            image_count+=1
     #retrning the appended list of processed image data
    return smooth_list

# %%
#image_smoothing_scikit('C:\\Users\\kavya\\Desktop\\Kavya\\preprocessing_wrappers\\scikit_opencv\\image_data',False,'denoise_wavelet',True)

# %% morphology
# %% morphology
def image_morphology_scikit(image_path,is_image,method,save_image=False):
  # if user selects a invalid method we raise an error
    if method not in['erosion','dilation']: 
         raise ValueError('Please input correct Operation. Valid inputs:''erosion','dilation')
    # if user inputs single image, check for only image formats and pass for further processing.
    #save_image= True will save the image file to the woring directory to view 
    if is_image == True:
        image = imread(image_path)
        if (image_path.endswith('.jpeg') or image_path.endswith('.png') or image_path.endswith('.jpg')): pass
        else: raise ValueError('please provide a valid image path for is_image=True')
        #image passes through the mentioned method for morphology 
        if method == 'erosion':
            morph_list = erosion(image, selem=None, out=None)
            if save_image==True:
                imsave('image_erosion.jpg', morph_list)
        elif method == 'dilation':
            morph_list = dilation(image, selem=None, out=None)
        if save_image==True:
            imsave('image_dilation.jpg', morph_list)
    #if is_image== False, loop continuous to process and check for only image format files for further processing
    elif is_image == False:
        if (image_path.endswith('.jpeg') or image_path.endswith('.png') or image_path.endswith('.jpg')): pass
        else: pass
        #creating new list to append all the processed content into morph_list=[]
        morph_list = []
        #extract_images function gets activated and images are read and stored into image_path_list
        image_path_list = extract_images(image_path)
        #image_count= 1 to intialize to 1 which will get incrimented everytime when a new data enters the list.
        image_count = 1
        for i in image_path_list:
            image_read = imread(i)
            #image passes through the mentioned method for morphology
            if method == 'erosion':
                err = erosion(image_read, selem=None, out=None)
                if save_image==True:
                    imsave('image_{}_erosion.jpg'.format(image_count),err)
                    print('saved')
                morph_list.append(err) 
            elif method == 'dilation':
                dilate = dilation(image_read, selem=None, out=None)
                if save_image==True:
                    imsave('image_{}_dilasion.jpg'.format(image_count),dilate)
                    print('saved')
                morph_list.append(dilate)
            else:
                print('wrong operation')
            #image_count gets incrimented everytime a new file gets processed.
            image_count+=1
    #retrning the appended list of processed image data        
    return morph_list
#%%
#image_morphology_scikit('C:\\Users\\kavya\\Desktop\\Kavya\\preprocessing_wrappers\\scikit_opencv\\image_data',False,'dilation',True)
# %%
def image_segmentation_scikit(image_path, is_image,save_image=False):
  # if user inputs single image, check for only image formats and pass for further processing
  #image passes through the mentioned method for segmentation 
  #save_image= True will save the image file to the woring directory to view 
    if is_image == True:
        #image = imread(image_path)
        if (image_path.endswith('.jpeg') or image_path.endswith('.png') or image_path.endswith('.jpg')): pass
        else: raise ValueError('please provide a valid image path for is_image=True')
        image_read = imread(image_path)
        image_read = rgb2gray(image_read)
        mask = image_read > filters.threshold_otsu(image_read)
        clean_border = segmentation.clear_border(mask).astype(np.int)
        seg_edges = segmentation.mark_boundaries(image_read, clean_border)
        if save_image==True:
            imsave('image_segmentation.jpg', seg_edges)
        edges=seg_edges
    #if is_image== False, loop continuous to process and check for only image format files for further processing
    elif is_image == False:
        if (image_path.endswith('.jpeg') or image_path.endswith('.png') or image_path.endswith('.jpg')): pass
        else: pass
        #creating new list to append all the processed content into edges=[]
        edges = []
        #extract_images function gets activated and images are read and stored into image_path_list
        image_path_list = extract_images(image_path)
        #image_count= 1 to intialize to 1 which will get incrimented everytime when a new data enters the list.
        image_count = 1
        #image passes through the mentioned method for segmentation
        #save_image= True will save the image file to the woring directory to view 
        for i in image_path_list:
            image_read = imread(i)
            image_read = rgb2gray(image_read)
            mask = image_read > filters.threshold_otsu(image_read)
            clean_border = segmentation.clear_border(mask).astype(np.int) 
            seg_edges = segmentation.mark_boundaries(image_read, clean_border)
            if save_image==True:
                imsave('image_{}_segmentation.jpg'.format(image_count),seg_edges)
                print('saved')
            edges.append(seg_edges)
            image_count+=1
    return edges
#%%
#image_segmentation_scikit('C:\\Users\\kavya\\Desktop\\Kavya\\preprocessing_wrappers\\scikit_opencv\\image_data\\elep.jpg',True,True)

# %%



from dltk_ai.image_preprocessor_files.opencv_preprocessor import *
from dltk_ai.image_preprocessor_files.scikit_image_preprocessor import *
import warnings

def image_transformation(datapath,is_image,method,library,save_image=False,show_image=False):
    if library == 'opencv':
        transformed = image_transformation_opencv(datapath=datapath,is_image=is_image,method=method,save_image=save_image,show_image=False)
    if library == 'scikit_image':
         if show_image==True:
             warnings.warn('there is no show_image feature for scikit_image. defaulting show_image to False')
         transformed = image_transformation_scikit(image_path=datapath,is_image=is_image,method=method,save_image=save_image)
    return transformed

def image_colorspace(datapath,is_image,method,library,save_image=False,show_image=True):
    if library=='opencv':
        r_color = image_colorspace_opencv(datapath=datapath, is_image=is_image, method=method, show_image=show_image,save_image=save_image)
        if show_image == True:
            warnings.warn('there is no show_image feature for scikit_image. defaulting show_image to False')
    if library == 'scikit_image':
        r_color = image_colorspace_scikit(image_path=datapath,is_image=is_image,method=method,save_image=save_image)
    return r_color

def image_resize(datapath,is_image, method, library, save_image=False, show_image= False,width=None,height=None):
    if library== 'opencv':
        r_resize= image_resize_opencv(datapath= datapath,is_image= is_image ,method= method,show_image=show_image, save_image= save_image,width= width, height= height)
    if library == 'scikit_image':
        if show_image== True:
            warnings.warn('There is no show_image feature for scikit_image. defaulting show_image to False')
        r_resize= image_resize_scikit(image_path= datapath,is_image= is_image ,method= method, save_image= save_image)
    return r_resize

def image_segmentation(datapath,is_image,library, save_image=False, show_image= False):
    if library== 'opencv':
        r_segmentaton= image_segmentation_opencv(datapath= datapath,is_image= is_image,show_image=show_image, save_image= save_image )
    if library == 'scikit_image':
        if show_image== True:
            warnings.warn('There is no show_image feature for scikit_image. defaulting show_image to False')
        r_segmentaton= image_segmentation_scikit(image_path = datapath,is_image= is_image, save_image= save_image)
    return r_segmentaton

def image_smoothing(datapath,is_image, method, library, save_image=False, show_image= False):
    if library== 'opencv':
        r_smoothing= image_smoothing_opencv(datapath= datapath,is_image=is_image,method=method,show_image=show_image, save_image= save_image )
    if library == 'scikit_image':
        if show_image== True:
            warnings.warn('There is no show_image feature for scikit_image. defaulting show_image to False')
        r_smoothing = image_smoothing_scikit(image_path= datapath,is_image= is_image, method=method,save_image= save_image)
    return r_smoothing

def image_morphology(datapath,is_image, method, library, save_image=False, show_image= False):
    r_morphology=[]
    if library== 'opencv':
        r_morphology= image_morphology_opencv(datapath= datapath,is_image=is_image,method=method,show_image=show_image, save_image= save_image )
    if library == 'scikit_image':
        if show_image== True:
            warnings.warn('There is no show_image feature for scikit_image. defaulting show_image to False')
        r_morphology= image_morphology_scikit(image_path= datapath,is_image= is_image, method=method,save_image= save_image)
    return r_morphology

if __name__ == '__main__':
    print("hello")

    a= image_morphology(r'C:\Users\Gowtham\Desktop\rohith\dog.jpg',True,'dilation','scikit_image',True)
    print(a)
